import json
import wget
import time
from pathlib import Path


def cerberus_metafit(gps_time, out_dir):
    cerberus_url = 'http://ws.mwatelescope.org/metadata/fits?obs_id='
    print(f'Downloading {gps_time} metafits')
    wget.download(f'{cerberus_url}{gps_time}', f'{out_dir}/{gps_time}.metafits')


if __name__ == "__main__":

    import argparse
    
    parser = argparse.ArgumentParser(description="""
            Download metafits files to find
            dipole flagging of tiles
            """)
    
    parser.add_argument(
            '--obs_ids', metavar='\b', default='./../../outputs/beam_pointings/ultimate_pointing_times.json', 
            help='Directory where json metadata files live. Default=./../../outputs/beam_pointings/ultimate_pointing_times.json')
    
    parser.add_argument(
            '--out_dir', metavar='\b', default='./../../outputs/beam_pointings/metafits/', 
            help='Directory where json metadata files live. Default=./../../outputs/beam_pointings/metafits/')
    
    args = parser.parse_args()
    obs_ids     = args.obs_ids
    out_dir     = Path(args.out_dir)
    
    out_dir.mkdir(parents=True, exist_ok=True)
    
    with open(obs_ids) as gps:
        gps_times = json.load(gps)['start_gps']
    
    for i in gps_times[10::10]:
        cerberus_metafit(i, out_dir)
        print('\n')
        time.sleep(10)   
        



import sys
import numpy as np
import healpy as hp
from null_test import rotate
from scipy.stats import median_absolute_deviation as mad

sys.path.append('../sat_ephemeris')
from sat_ids import norad_ids

def good_maps(raw_tile):
    '''Here, we extract "good satellie" data from the tile_maps_raw 
    generated by tile_maps.py. We also normalize the maps by
    the reference beam'''

    f_name, _ = raw_tile.name.split('.')
    tile, ref, _, _ = f_name.split('_')
    
    # Load reference FEE model
    # Rotate the fee models by -pi/2 to move model from spherical (E=0) to Alt/Az (N=0)
    ref_fee_model = np.load(ref_model, allow_pickle=True)
    if 'XX' in tile:
        ref_fee = ref_fee_model['XX']
        rotated_fee = rotate(nside, angle=-(1*np.pi)/2.0,healpix_array=ref_fee)
    else:
        ref_fee = ref_fee_model['YY']
        rotated_fee = rotate(nside, angle=-(1*np.pi)/2.0,healpix_array=ref_fee)
   
    # load data from map .npz file
    tile_data   = np.load(raw_tile, allow_pickle=True)
    tile_data   = {key:tile_data[key].item() for key in tile_data}
    mwa_map     = tile_data['mwa_map']  
    
    tile_maps_norm = {p:[] for p in pointings}
    mwa_maps_norm = {p:[] for p in pointings}
     
    for p in pointings:
        
        # mwa map
        mwa_map_good = [[] for pixel in range(hp.nside2npix(nside))]
        
        for sat in good_sats:

            for pix in range(hp.nside2npix(nside)):

                mwa_map_good[pix].extend(mwa_map[p][sat][pix])

        mwa_maps_norm[p].extend(mwa_map_good)
        
    # Save map arrays to npz file
    np.savez_compressed(f'{out_dir}/{tile}_{ref}_tile_maps.npz', **mwa_maps_norm)


if __name__=='__main__':
    
    import argparse
    from pathlib import Path
    import concurrent.futures

    parser = argparse.ArgumentParser(description="""
        Extract the data from a list of 'good satellites' from the raw tile maps made my tile_maps.py. 
        Divide the tile/ref beams and apply the FEE ref model.
        """)
    
    parser.add_argument('--out_dir', metavar='\b', default='./../../outputs/tile_maps/tile_maps_norm/',
            help='Output directory. Default=./../../outputs/tile_maps/tile_maps_norm/')
    parser.add_argument('--map_dir', metavar='\b', default='./../../outputs/tile_maps/tile_maps_raw',
            help='Raw Map directory. Default=./../../outputs/tile_maps/tile_maps_raw')
    parser.add_argument('--ref_model', metavar='\b', default='../../outputs/reproject_ref/ref_dipole_models.npz',
            help='Healpix reference FEE model file. default=../../outputs/reproject_ref/ref_dipole_models.npz')
    parser.add_argument('--nside', metavar='\b', type=int,  default=32,help='Healpix Nside. Default = 32')
    
    args = parser.parse_args()
    
    out_dir     = Path(args.out_dir)
    map_dir     = Path(args.map_dir)
    ref_model   = Path(args.ref_model)
    nside       = args.nside
    
    # make outpud dir
    out_dir.mkdir(parents=True, exist_ok=True)
    
    # Good sats from which to make plots
    good_sats = [
            25338, 25982, 25984, 25985,
            28654, 40086, 40087,
            40091, 41179, 41180,
            41182, 41183, 41184,
            41185, 41187, 41188,
            41189, 44387
            ]

    # use all satellites
    #good_sats = list(norad_ids.values())

    # list of beam pointings
    pointings = ['0','2','4','41']
   
    # list of all raw tile maps
    map_files = [item for item in map_dir.glob('*.npz')]

    # Parallization magic happens here
    with concurrent.futures.ProcessPoolExecutor() as executor:
        results = executor.map(good_maps, map_files)

#    good_maps(map_files[0])









import sys
import numpy as np
import healpy as hp
from mwa_pb.mwa_sweet_spots import all_grid_points
from scipy.stats import median_absolute_deviation as mad

def plot_healpix(data_map=None,sub=None,title=None,vmin=None,vmax=None,cmap=None):
    '''Yeesh do some healpix magic to plot the thing'''
    
    # Healpix plotting script adapted from Dr. Jack Line's code
    # https://github.com/JLBLine/MWA_ORBCOMM
    
    # Disable cryptic healpy warnings. Can't figure out where they originate
    import warnings
    warnings.filterwarnings("ignore", category=RuntimeWarning) 
    
    if vmin == None:
        if cmap == None:
            half_sky = hp.orthview(
                    map=data_map,coord='E',
                    half_sky=True,xsize=400,
                    title=title,rot=(0,90,180),
                    sub=sub,notext=True, return_projected_map=True)
        else:
            half_sky = hp.orthview(
                    map=data_map,coord='E',
                    half_sky=True,xsize=400,
                    title=title,rot=(0,90,180), sub=sub,cmap=cmap,
                    notext=True,return_projected_map=True)
    else:
        if cmap == None:
            half_sky = hp.orthview(
                    map=data_map,coord='E',
                    half_sky=True,xsize=400,rot=(0,90,180),
                    title=title,sub=sub,min=vmin,max=vmax,
                    notext=True,return_projected_map=True)
        else:
            half_sky = hp.orthview(
                    map=data_map,coord='E',
                    half_sky=True,xsize=400,rot=(0,90,180),
                    title=title,sub=sub,min=vmin,max=vmax,
                    cmap=cmap,notext=True,return_projected_map=True)

    hp.graticule(dpar=10,coord='E',color='k',alpha=0.3,dmer=45)
   
    # Altitude grid
    hp.projtext(00.0*(np.pi/180.0), 225.0*(np.pi/180), '0', coord='E')
    hp.projtext(30.0*(np.pi/180.0), 225.0*(np.pi/180), '30', coord='E')
    hp.projtext(60.0*(np.pi/180.0), 225.0*(np.pi/180), '60', coord='E')

    # Azimuth grid
    hp.projtext(75.0*(np.pi/180.0), 000.0*(np.pi/180.0), r'$0^\circ$',   coord='E',color='w', fontsize=10, verticalalignment='top')
    hp.projtext(75.0*(np.pi/180.0), 090.0*(np.pi/180.0), r'$90^\circ$',  coord='E',color='w', fontsize=10, horizontalalignment='right')
    hp.projtext(75.0*(np.pi/180.0), 180.0*(np.pi/180.0), r'$180^\circ$', coord='E',color='w', fontsize=10, verticalalignment='bottom')
    hp.projtext(75.0*(np.pi/180.0), 270.0*(np.pi/180.0), r'$270^\circ$', coord='E',color='w', fontsize=10, horizontalalignment='left')
    
    # NSEW 
    hp.projtext(90.0*(np.pi/180.0), 000.0*(np.pi/180.0), r'$N  $', coord='E',color='k', fontsize=14, verticalalignment='bottom')
    hp.projtext(90.0*(np.pi/180.0), 090.0*(np.pi/180.0), r'$E  $', coord='E',color='k', fontsize=14, horizontalalignment='left')
    hp.projtext(90.0*(np.pi/180.0), 180.0*(np.pi/180.0), r'$S  $', coord='E',color='k', fontsize=14, verticalalignment='top')
    hp.projtext(90.0*(np.pi/180.0), 270.0*(np.pi/180.0), r'$W  $', coord='E',color='k', fontsize=14, horizontalalignment='right')


def plt_good_maps(f):
    
    f_name, _ = f.name.split('.')
    tile, ref, _, _ = f_name.split('_')
   
    pointings = ['0','2','4','41']


    # load data from map .npz file
    tile_data = np.load(f, allow_pickle=True)
    
    for p in pointings:
   
        ## find the pointing center in radians
        #pointing_center_az = np.radians(all_grid_points[int(p)][1])
        #pointing_center_za = np.radians(all_grid_points[int(p)][3])
        #
        ## convert it to a healpix vector
        #pointing_vec = hp.ang2vec(pointing_center_za, pointing_center_az)

        ## find all healpix indices within 10 degrees of pointing center
        #ipix_disc = hp.query_disc(nside=nside, vec=pointing_vec, radius=np.radians(10))
        
        # healpix meadian map
        tile_map_med = np.asarray([(np.nanmedian(i) if i != [] else np.nan ) for i in tile_data[p]])
       
        ## find the max value within 10 degrees of pointing center
        #ipix_max = np.nanmax(tile_map_med[ipix_disc])
        #
        ## scale map such that the above max is set to 0dB 
        #tile_map_scaled = np.asarray([(i - ipix_max) for i in tile_map_med])
        
        fig = plt.figure(figsize=(8,10))
        fig.suptitle(f'Good Map: {tile}/{ref} @ {p}', fontsize=16)
        plot_healpix(data_map=tile_map_med, sub=(1,1,1), cmap=jade, vmin=-50, vmax=0)
        plt.savefig(f'{out_dir}/{p}/tile_maps/{tile}_{ref}_{p}_good_map.png',bbox_inches='tight')
        plt.close()
        
        # Plot MAD 
        tile_map_mad = []
        for j in tile_data[p]:
            if j != []:
                j = np.asarray(j)
                j = j[~np.isnan(j)]
                tile_map_mad.append(mad(j))
            else:
                tile_map_mad.append(np.nan)

        vmin = np.nanmin(tile_map_mad)
        vmax = np.nanmax(tile_map_mad)

        fig = plt.figure(figsize=(8,10))
        fig.suptitle(f'Good Map MAD: {tile}/{ref} @ {p}', fontsize=16)
        plot_healpix(data_map=np.asarray(tile_map_mad),sub=(1,1,1), cmap=jade, vmin=vmin, vmax=vmax)
        plt.savefig(f'{out_dir}/{p}/tile_errors/{tile}_{ref}_{p}_good_map_errors.png',bbox_inches='tight')
        plt.close()
        

        # Plot counts in pix
        tile_map_counts = [len(np.array(i)[~np.isnan(i)]) for i in tile_data[p]]
        
        fig = plt.figure(figsize=(8,10))
        fig.suptitle(f'Good Map Counts: {tile}/{ref} @ {p}', fontsize=16)
        plot_healpix(data_map=np.asarray(tile_map_counts),sub=(1,1,1), cmap=jade, vmin=0, vmax=80)
        plt.savefig(f'{out_dir}/{p}/tile_counts/{tile}_{ref}_{p}_good_map_counts.png',bbox_inches='tight')
        plt.close()


if __name__=='__main__':
    
    import argparse
    import matplotlib
    matplotlib.use('Agg')
    from pathlib import Path
    import concurrent.futures
    import matplotlib.pyplot as plt
    
    import sys
    sys.path.append('../decode_rf_data')
    from colormap import spectral, jade, kelp
    
    # Custom spectral colormap
    jade, _ = jade()
    

    parser = argparse.ArgumentParser(description="""
        Plot healpix maps of normalized tile maps
        """)
    
    parser.add_argument('--out_dir', metavar='\b', default='./../../outputs/tile_maps/good_maps',help='Output directory. Default=./../../outputs/tile_maps/good_maps')
    parser.add_argument('--map_dir', metavar='\b', default='./../../outputs/tile_maps/tile_maps_norm',help='Output directory. Default=./../../outputs/tile_maps/tile_maps_norm')
    parser.add_argument('--nside', metavar='\b', type=int,  default=32,help='Healpix Nside. Default = 32')
    
    args = parser.parse_args()
    
    out_dir = Path(args.out_dir)
    map_dir = Path(args.map_dir)
    nside   = args.nside
    
    pointings = ['0','2','4','41']
    
    map_files = [item for item in map_dir.glob('*.npz')]

    # Create output directory tree
    for p in pointings:
        Path(f'{out_dir}/{p}/tile_maps/').mkdir(parents=True, exist_ok=True)
        Path(f'{out_dir}/{p}/tile_counts/').mkdir(parents=True, exist_ok=True)
        Path(f'{out_dir}/{p}/tile_errors/').mkdir(parents=True, exist_ok=True)

    # Parallization magic happens here
    with concurrent.futures.ProcessPoolExecutor() as executor:
        results = executor.map(plt_good_maps, map_files)
    




import time
import numpy as np
from colormap import spectral
import matplotlib
# Force matplotlib to not use X-Server backend
matplotlib.use('Agg')
import matplotlib.pyplot as plt


def read_data(filename):
    '''Converts raw rf data from the RF Explorer
    to a power and time array. The raw data is 
    saved in binary format. Time is in UNIX format.
    '''
   
    try:
        with open(filename, 'rb') as f:
            next(f)
            lines = f.readlines()
            
            times = []
            data_lines = []

            for line in lines:
                time, data = line.split('$Sp'.encode())
                times.append(time.decode())
                
                # List converts bytes to list of bytes
                # The last two charachters are excluded - Newline char
                data_lines.append(list(data[:-2]))
            
            # I have no idea where the (-1/2) factor comes from
            # Remenant of Jared Rasti's code. Seems to work
            # Must be something to do with how data is written from RF
            # The (-1/2) converts an unsigned byte to a real value
            power = np.single(np.asarray(data_lines) * (-1 / 2))
            times = np.double(np.asarray(times))
            
            return (power, times)
    except Exception:
        print(f'Missing {filename}, moving on...')


def tile_names():
    '''A list of tile reference and MWA tile names '''
    tiles = [
            'rf0XX', 'rf0YY', 'rf1XX', 'rf1YY',
            'S06XX', 'S06YY', 'S07XX', 'S07YY',
            'S08XX', 'S08YY', 'S09XX', 'S09YY',
            'S10XX', 'S10YY', 'S12XX', 'S12YY',
            'S29XX', 'S29YY', 'S30XX', 'S30YY',
            'S31XX', 'S31YY', 'S32XX', 'S32YY',
            'S33XX', 'S33YY', 'S34XX', 'S34YY',
            'S35XX', 'S35YY', 'S36XX', 'S36YY'
            ]
    return tiles


def plot_waterfall(power, times, name, n_axes=True):
    '''Using power and times array, creates a waterfall
    plot depicting satellite passes in the raw rf data.
    By default this function will use n_axes = True to
    convert unix times to human readable HH:MM local time
    for the y-axis, and use the rf freqencies for the x-azis.
    
    If n_axes=False, will plot waterfall with time step
    vs freq channel (0-112)
    '''

    power_median = np.median(power)
    image = power - power_median
    # setting dynamic range of waterfall to be 30 dB above the median
    vmin = 0
    vmax = 30

    # Custom spectral colormap
    cmap = spectral()
    
    plt.style.use('dark_background')
    fig = plt.figure(figsize = (7,10))
    ax = fig.add_axes([0.12, 0.1, 0.72, 0.85])
    im = ax.imshow(image, vmin=vmin, vmax=vmax, interpolation='none', cmap=cmap)
    cax = fig.add_axes([0.88, 0.1, 0.03, 0.85])
    fig.colorbar(im, cax=cax)
    ax.set_aspect('auto')
    ax.set_title(f'Waterfall Plot: {name}')

    if n_axes == True:
    
        # Number of time steps on y-axis
        number_t = 5
        t_step = int(len(times)/(number_t-1))
        times = list(times)
        times = times[::t_step]
        
        t_tz = []
        
        # Convert UNIX time to local HH:MM time
        for i in range(len(times)):
            
            perth_t = float(times[i])+28800 #28800=+8GMT @ PERTH
            hms = time.strftime('%H:%M', time.gmtime(perth_t))
            t_tz.append(hms)

        # Frequency: x-axis
        start_freq = 137.15
        stop_freq = 138.55

        # X-axis stuff
        x_ax = image.shape[1]
        freqs = np.arange(start_freq, stop_freq, 0.25)
        x_ticks = np.arange(0, x_ax, (0.25/0.0125)) #.0125MHz/ch
        ax.set_xticks(x_ticks)
        ax.set_xticklabels(freqs)
        ax.set_xlabel('Freq [MHz]')

        # Y-axis stuff
        y_ax = image.shape[0]
        y_ticks = np.arange(0, y_ax, t_step)
        ax.set_yticks(y_ticks)
        ax.set_yticklabels(t_tz)
        ax.set_ylabel('Time [HH:MM]')

    else:
        ax.set_xlabel('Freq Channel')
        ax.set_ylabel('Time Step')
    
    return plt





import sys
import numpy as np
import healpy as hp
import scipy.optimize as opt
import numpy.polynomial.polynomial as poly
from mwa_pb.mwa_sweet_spots import all_grid_points

sys.path.append('../decode_rf_data')
from colormap import spectral, jade, kelp

# Custom spectral colormap
jade, _ = jade()


def plot_healpix(data_map=None, fig=None, sub=None,title=None,vmin=None,vmax=None,cmap=None, cbar=True):
    '''Yeesh do some healpix magic to plot the thing'''
    
    # Disable cryptic healpy warnings. Can't figure out where they originate
    import warnings
    warnings.filterwarnings("ignore", category=RuntimeWarning) 
    
    hp.delgraticules()
    hp.orthview(
            map=data_map,coord='E', fig=fig,
            half_sky=True, rot=(0,90,180), xsize=1200,
            title=title,sub=sub,min=vmin,max=vmax,
            cmap=cmap,notext=True, hold=True, cbar=cbar, return_projected_map=False)

    hp.graticule(dpar=10,coord='E',color='k',alpha=0.7,dmer=45, lw=0.4, ls=':')
   
    # Altitude grid
    hp.projtext(00.0*(np.pi/180.0), 225.0*(np.pi/180),'0',  color='k', coord='E', fontsize=6, fontweight='light')
    hp.projtext(30.0*(np.pi/180.0), 225.0*(np.pi/180),'30', color='k', coord='E', fontsize=6, fontweight='light')
    hp.projtext(60.0*(np.pi/180.0), 225.0*(np.pi/180),'60', color='k', coord='E', fontsize=6, fontweight='light')

    # NSEW 
    hp.projtext(80.0*(np.pi/180.0), 000.0*(np.pi/180.0), r'$N  $', coord='E', color='w', fontweight='light',  verticalalignment='top')
    hp.projtext(80.0*(np.pi/180.0), 090.0*(np.pi/180.0), r'$E  $', coord='E', color='w', fontweight='light',  horizontalalignment='right')
    hp.projtext(80.0*(np.pi/180.0), 180.0*(np.pi/180.0), r'$S  $', coord='E', color='w', fontweight='light',  verticalalignment='bottom')
    hp.projtext(80.0*(np.pi/180.0), 270.0*(np.pi/180.0), r'$W  $', coord='E', color='w', fontweight='light',  horizontalalignment='left')


def beam_maps(f):
    '''Returns pairs of [AUT,FEE] maps, for each pointing'''
    f_name, _ = Path(f).name.split('.')
    t_name, r_name, _, _ = f_name.split('_')

    #pointings = ['0','2','4','41']
    pointings = ['0','2','4']
    
    maps = []

    # load data from map .npz file
    tile_map = np.load(f, allow_pickle=True)
    fee_m = np.load(fee_map, allow_pickle=True)
    
    for p in pointings:

        tile = tile_map[p]

        if 'XX' in t_name:
            fee = fee_m[p][0]
        else:
            fee = fee_m[p][1]
        

        # Visualize the tile map and diff map
        # healpix meadian map
        tile_med = np.asarray([(np.nanmedian(j) if j != [] else np.nan ) for j in tile])
        
        residuals = tile_med - fee
        residuals[np.where(fee < -30)] = np.nan
        residuals[np.where(tile_med == np.nan)] = np.nan
       
        maps.append([tile_med, residuals])
        
    return maps


def plt_grid(tile_name, ref_name):
    # This is an Awesome plot

    f_xx = f'{map_dir}/{tile_name}XX_{ref_name}XX_tile_maps.npz'
    f_yy = f'{map_dir}/{tile_name}YY_{ref_name}YY_tile_maps.npz'
    
    maps_xx = beam_maps(f_xx) 
    maps_yy = beam_maps(f_yy)
    
    nice_fonts = {
            # Use LaTeX to write all text
            #"text.usetex": True,
            "font.family": "sans-serif",
            # Use 10pt font in plots, to match 10pt font in document
            "axes.labelsize": 8,
            "axes.titlesize": 9,
            "font.size": 8,
            # Make the legend/label fonts a little smaller
            "legend.fontsize": 6,
            "xtick.labelsize": 8,
            "ytick.labelsize": 8,
            }
    plt.rcParams.update(nice_fonts)
    #plt.rcParams['grid.color'] = '#cccccc'
    #plt.rcParams['grid.linestyle'] = ':'
    #plt.rcParams['grid.linewidth'] = 0.3
    #plt.style.use('seaborn')
   
    fig1 = plt.figure(figsize=(7.6,9.0))

    ax1  = fig1.add_axes([0.01, 0.75, 0.29, 0.22])
    plot_healpix(data_map=maps_xx[0][0],  fig=fig1, title=fr'($i$) {tile_name}XX [Zenith]', cmap=jade, vmin=-50, vmax=0,  cbar=False)
    ax2  = fig1.add_axes([0.30, 0.75, 0.29, 0.22])
    plot_healpix(data_map=maps_xx[1][0],  fig=fig1, title=fr'($ii$) {tile_name}XX [2]', cmap=jade, vmin=-50, vmax=0,  cbar=False)
    ax3  = fig1.add_axes([0.60, 0.75, 0.29, 0.22])
    plot_healpix(data_map=maps_xx[2][0],  fig=fig1, title=fr'($iii$) {tile_name}XX [4]', cmap=jade, vmin=-50, vmax=0,  cbar=False)
    ax3 = plt.gca()
    image = ax3.get_images()[0]
    cax1 = fig1.add_axes([0.91, 0.75, 0.015, 0.22])
    cbar1 = fig1.colorbar(image, cax=cax1, label='Power [dB]')
    
    
    
    ax4  = fig1.add_axes([0.01, 0.50, 0.29, 0.22])
    plot_healpix(data_map=maps_xx[0][1],  fig=fig1, title=fr'($iv$) {tile_name}XX - FEE [Zenith]', cmap='RdYlGn', vmin=-5, vmax=5,  cbar=False)
    ax5  = fig1.add_axes([0.30, 0.50, 0.29, 0.22])
    plot_healpix(data_map=maps_xx[1][1],  fig=fig1, title=fr'($v$) {tile_name}XX - FEE [2]',      cmap='RdYlGn', vmin=-5, vmax=5,  cbar=False)
    ax6  = fig1.add_axes([0.60, 0.50, 0.29, 0.22])
    plot_healpix(data_map=maps_xx[2][1],  fig=fig1, title=fr'($vi$) {tile_name}XX - FEE [4]',      cmap='RdYlGn', vmin=-5, vmax=5,  cbar=False)
    ax6 = plt.gca()
    image = ax6.get_images()[0]
    cax2 = fig1.add_axes([0.91, 0.50, 0.015, 0.22])
    cbar2 = fig1.colorbar(image, cax=cax2, label='Power [dB]')
    
    ax7  = fig1.add_axes([0.01, 0.25, 0.29, 0.22])
    plot_healpix(data_map=maps_yy[0][0],  fig=fig1, title=fr'($vii$) {tile_name}YY [Zenith]', cmap=jade, vmin=-50, vmax=0,  cbar=False)
    ax8  = fig1.add_axes([0.30, 0.25, 0.29, 0.22])
    plot_healpix(data_map=maps_yy[1][0],  fig=fig1, title=fr'($viii$) {tile_name}YY [2]', cmap=jade, vmin=-50, vmax=0,  cbar=False)
    ax9  = fig1.add_axes([0.60, 0.25, 0.29, 0.22])
    plot_healpix(data_map=maps_yy[2][0],  fig=fig1, title=fr'($ix$) {tile_name}YY [4]', cmap=jade, vmin=-50, vmax=0,  cbar=False)
    ax9 = plt.gca()
    image = ax9.get_images()[0]
    cax3 = fig1.add_axes([0.91, 0.25, 0.015, 0.22])
    cbar3 = fig1.colorbar(image, cax=cax3, label='Power [dB]')
    
    ax10 = fig1.add_axes([0.01, 0.0, 0.29, 0.22])
    plot_healpix(data_map=maps_yy[0][1], fig=fig1, title=fr'($x$) {tile_name}YY - FEE [Zenith]', cmap='RdYlGn', vmin=-5, vmax=5,  cbar=False)
    ax11 = fig1.add_axes([0.30, 0.0, 0.29, 0.22])
    plot_healpix(data_map=maps_yy[1][1], fig=fig1, title=fr'($xi$) {tile_name}YY - FEE [2]',      cmap='RdYlGn', vmin=-5, vmax=5,  cbar=False)
    ax12 = fig1.add_axes([0.60, 0.0, 0.29, 0.22])
    plot_healpix(data_map=maps_yy[2][1], fig=fig1, title=fr'($xii$) {tile_name}YY - FEE [4]',      cmap='RdYlGn', vmin=-5, vmax=5,  cbar=False)
    ax12 = plt.gca()
    image = ax12.get_images()[0]
    cax4 = fig1.add_axes([0.91, 0.0, 0.015, 0.22])
    cbar4 = fig1.colorbar(image, cax=cax4, label='Power [dB]')
    
    
    plt.savefig(f'{out_dir}/{tile_name}_{ref_name}_maps.pdf', bbox_inches='tight', dpi=420)
    plt.close()


if __name__=='__main__':
    
    import argparse
    import numpy as np
    from pathlib import Path
    import concurrent.futures
    import matplotlib
    matplotlib.use('Agg')
    import matplotlib.pyplot as plt
    import matplotlib.gridspec as gs
    from mpl_toolkits.axes_grid1 import make_axes_locatable
    from scipy.stats import median_absolute_deviation as mad

    import sys
    sys.path.append('../decode_rf_data')
    from colormap import spectral
    
    # Custom spectral colormap
    cmap = spectral()
    
    parser = argparse.ArgumentParser(description="""
        Plot healpix map of reference data
        """)
    
    parser.add_argument('--out_dir', metavar='\b', default='../../outputs/paper_plots/tile_grids/',
            help='Output directory. Default=../../outputs/paper_plots/tile_grids')
    parser.add_argument('--map_dir', metavar='\b', default='../../outputs/tile_maps/tile_maps_norm/',
            help='Tile map directory. Default=../../outputs/tile_maps/tile_maps_norm/')
    parser.add_argument('--fee_map', metavar='\b', default='../../outputs/tile_maps/FEE_maps/mwa_fee_beam.npz',
            help='Healpix FEE map of mwa tile. default=../../outputs/tile_maps/FEE_maps/mwa_fee_beam.npz')
    parser.add_argument('--nside', metavar='\b',type=int, default=32,help='Healpix Nside. Default = 32')
    
    
    args = parser.parse_args()
    
    out_dir     = Path(args.out_dir)
    map_dir     = Path(args.map_dir)
    fee_map     = args.fee_map
    nside       = args.nside
    
    # make output dir if it doesn't exist
    out_dir.mkdir(parents=True, exist_ok=True)

    tiles = [
            'S06', 'S07', 'S08', 'S09',
            'S10', 'S12', 'S29', 'S30',
            'S31', 'S32', 'S33', 'S34',
            'S35', 'S36'
            ]

    refs = ['rf0', 'rf1']
    
    #for r in refs:
    #    for t in tiles:
    #        plt_grid(t, r)

    plt_grid('S07','rf1')





import math
import time
import numpy as np
from pathlib import Path
from scipy import interpolate
from scipy.signal import savgol_filter


def savgol_interp(ref, tile, savgol_window_1 =None,savgol_window_2=None, polyorder=None, interp_type=None, interp_freq=None):
    """Smooth and interpolate the power array

    Interpolate to a given frequency which 
    makes the dimentions of the power arrays
    from the reference antenna and the tile equal,
    allowing a one to one comparison at corresponding times.

    Two level of savgol filter applied, first to capture
    deep nulls + small structure, and second level to
    smooth over noise.

    
    Args:
        ref:            Path to reference data file
        tile:           Path to tile data file
        savgol_window_1:  Window size of savgol filer. Must be odd. Default = 151
        savgol_window_2:  Window size of savgol filer. Must be odd. Default = 151
        polyorder:      Order of polynomial to fit to savgol_window. Default = 1
        interp_type:    Type of interpolation. Ex: 'cubic', 'linear'. Default = cubic
        interp_freq:    The freqency to which power array is interpolated. Default = 6 Hz

    Returns:
        ref_p_aligned:  Aligned reference power array
        tile_p_aligned: Aligned tile power array
        time_array:     Time array corresponding to power arrays
    """

    # Import custom rf_data module
    import sys
    sys.path.append('../decode_rf_data')
    from rf_data import read_data
    
    # Read time and power arrays from data files
    ref_p, ref_t = read_data(ref)
    tile_p, tile_t = read_data(tile)

    # Data is recorded at a range of frequencies, depending on the RF Explorer version
    # The Old models recoreded at 6-7Hz, while the new ones record at 8.5-9.2Hz.
    # To get around this, we interpolate the data to a desired freqeuncy.
    
    
    # Using the start and stop times, we create an array of times at which to 
    # evaluate our interpolated data
    # Round up/down to nearest integer of time
    start_time = math.ceil(max(ref_t[0], tile_t[0]))
    stop_time = math.floor(min(ref_t[-1], tile_t[-1]))
    
    # Array of times at which to evaluate the interpolated data
    time_array = np.arange(start_time, stop_time, (1 / interp_freq))
    
    f = interpolate.interp1d(ref_t, ref_p, axis=0, kind=interp_type)
    g = interpolate.interp1d(tile_t, tile_p, axis=0, kind=interp_type)
    
    # New power array, evaluated at the desired frequency
    ref_p_aligned = f(time_array)
    tile_p_aligned = g(time_array)
   
    # Savgol level 1. Capture nulls / small scale structure
    ref_p_aligned = savgol_filter(ref_p_aligned, savgol_window_1, polyorder, axis=0)
    tile_p_aligned = savgol_filter(tile_p_aligned, savgol_window_1, polyorder, axis=0)
    
    # Savgol level 2. Smooth noise
    ref_p_aligned = savgol_filter(ref_p_aligned,   savgol_window_2, polyorder, axis=0)
    tile_p_aligned = savgol_filter(tile_p_aligned, savgol_window_2, polyorder, axis=0)
    
    return (ref_t, ref_p, tile_t, tile_p, ref_p_aligned, tile_p_aligned, time_array)


if __name__ == '__main__':
    '''Plot a single channel to illustrate the savgol + interpolation
    
    Plots channel 59 of the sample date. This is the center 
    channel of the bright weather satellite seen in the bottom 
    right corner of the sample waterfall plots.'''
    
    import matplotlib
    # Enable x-window
    matplotlib.use('Agg')
    import matplotlib.pyplot as plt
    

    ch =59
    ref_t, ref_p, tile_t, tile_p, ref_p_aligned, tile_p_aligned, time_array = savgol_interp(
            './../../data/rf_data/rf0XX/2019-10-10/rf0XX_2019-10-10-02:30.txt',
            './../../data/rf_data/S06XX/2019-10-10/S06XX_2019-10-10-02:30.txt',
            savgol_window_1 =11,
            savgol_window_2 =15,
            polyorder=2,
            interp_type='cubic',
            interp_freq=1
            )



    # Sample align plot
    plt.style.use('seaborn')
    plt.rcParams["figure.figsize"] = (9,6)
    
    time_array = (time_array - time_array[0])/60
    ref_t = (ref_t - ref_t[0])/60
    tile_t = (tile_t - tile_t[0])/60


    plt.plot(time_array,tile_p_aligned[::, ch], color='#e23a4e', alpha=0.9, label='aut savgol')
    plt.scatter(tile_t, tile_p[::, ch], color='#f78b51',marker='.', alpha=0.6, label='aut tile')
    
    plt.plot(time_array,ref_p_aligned[::, ch], color='#252b40', alpha=0.9, label='ref savgol')	
    plt.scatter(ref_t, ref_p[::, ch], color='#6a82bb',marker='.', alpha=0.6, label='ref tile')

    leg = plt.legend(loc="upper left", frameon=True)
    leg.get_frame().set_facecolor('white')
    for l in leg.legendHandles:
        l.set_alpha(1)

   
    plt.ylim(-110, -20)
    plt.ylabel('Power [dBm]')
    plt.xlabel('Time [min]')
    plt.tight_layout()
    Path('../../outputs/align_data/').mkdir(parents=True, exist_ok=True)
    plt.savefig('../../outputs/align_data/align_data_test.png', dpi=300)





import json
import numpy as np
import seaborn as sns
from pathlib import Path
import matplotlib.pyplot as plt
    
import sys
sys.path.append('../sat_channels')
from window_chan_map import time_tree



def point_int(tile, start_date, stop_date, point_0, point_2, point_4, point_41, data_dir):
    """Calculate integration at each pointing for every tile"""
    
    # increment this by 0.5, for every obs which is in point_list. 
    # 0.5 = 30 min
    p_0     = 0
    p_2     = 0
    p_4     = 0
    p_41    = 0

    # dates: list of days
    # date_time = list of 30 min observation windows
    dates, date_time = time_tree(start_date, stop_date)
    for day in range(len(dates)):
        for window in range(len(date_time[day])):
            f_name = f'{tile}_{date_time[day][window]}.txt'
            f_path = Path(f'{data_dir}/{tile}/{dates[day]}/{f_name}')
            if f_path.is_file():
                if date_time[day][window] in point_0:
                    p_0 += 0.5
                elif date_time[day][window] in point_2:
                    p_2 += 0.5
                elif date_time[day][window] in point_4:
                    p_4 += 0.5
                elif date_time[day][window] in point_41:
                    p_41 += 0.5
                else:
                    pass
    return [p_0, p_2, p_4, p_41]


def plt_pointing_hist(a, b, c, fig, int_dict=None, tile=None, x_ticks=None):

    ax = fig.add_subplot(a,b,c)

    y_max = max(i for v in int_dict.values() for i in v)
    int_list = int_dict[tile]
    x = range(len(int_list))
    leg = int_list

    pal = sns.cubehelix_palette(len(int_list), start=.7, rot=-.7, dark=.4, reverse=True)
    barplot = plt.bar(x, int_list, color=sns.color_palette(pal), edgecolor='black')
    
    def autolabel(rects):
        for idx,rect in enumerate(barplot):
            height = rect.get_height()
            ax.text(rect.get_x() + rect.get_width()/2., height, 
                    leg[idx], ha='center', va='bottom', rotation=0)
    
    autolabel(barplot)
    
    # these are matplotlib.patch.Patch properties
    props = dict(boxstyle='round', facecolor='wheat', alpha=0.5)

    # place a text box in upper left in axes coords
    ax.text(0.65, 0.95, f'{tile}', transform=ax.transAxes, fontsize=10,
        verticalalignment='top', bbox=props)
   
    plt.xlim(-0.5,len(int_list)-0.5)
    ax.set_yticklabels([])
    
    ax.set_xticklabels([])
    ax.set_ylim(0, 1.1*y_max)
    

    return ax



if __name__=='__main__':

    import sys
    import argparse
    sys.path.append('../decode_rf_data')
    from rf_data import tile_names
    
    parser = argparse.ArgumentParser(description="""
        Determine total integration time, per pointing, for all tiles
        """)

    parser.add_argument('--data_dir', metavar='\b', help='Data directory')
    parser.add_argument(
            '--out_dir', metavar='\b', default='./../../outputs/beam_pointings/',
            help='Output directory. Default=./../../outputs/beam_pointings/')
    
    args = parser.parse_args()
   
    data_dir    = Path(args.data_dir)
    out_dir     = Path(args.out_dir)


    # Tile names
    tiles  = tile_names()

    # lists of obs at each pointings
    with open(f'{out_dir}/obs_pointings.json') as table:
        data = json.load(table)
        start_date  = data['start_date']
        stop_date   = data['stop_date']
        point_0     = data['point_0']
        point_2     = data['point_2']
        point_4     = data['point_4']
        point_41    = data['point_41']

    tile_integration = {}
    for tile in tiles:
        p_integration = point_int(tile, start_date, stop_date, point_0, point_2, point_4, point_41, data_dir)
        tile_integration[f'{tile}'] = p_integration


    plt.style.use('seaborn')
    fig = plt.figure(figsize=(14,10))

    ax01 = plt_pointing_hist(4,8,1, fig, int_dict=tile_integration, tile=tiles[0])
    ax02 = plt_pointing_hist(4,8,2, fig, int_dict=tile_integration, tile=tiles[4])
    ax03 = plt_pointing_hist(4,8,3, fig, int_dict=tile_integration, tile=tiles[5])
    ax04 = plt_pointing_hist(4,8,4, fig, int_dict=tile_integration, tile=tiles[6])
    ax05 = plt_pointing_hist(4,8,5, fig, int_dict=tile_integration, tile=tiles[7])
    ax06 = plt_pointing_hist(4,8,6, fig, int_dict=tile_integration, tile=tiles[8])
    ax07 = plt_pointing_hist(4,8,7, fig, int_dict=tile_integration, tile=tiles[9])
    ax08 = plt_pointing_hist(4,8,8, fig, int_dict=tile_integration, tile=tiles[10])
    ax09 = plt_pointing_hist(4,8,9, fig, int_dict=tile_integration, tile=tiles[1])
    ax10 = plt_pointing_hist(4,8,10, fig, int_dict=tile_integration, tile=tiles[11])
    ax11 = plt_pointing_hist(4,8,11, fig, int_dict=tile_integration, tile=tiles[12])
    ax12 = plt_pointing_hist(4,8,12, fig, int_dict=tile_integration, tile=tiles[13])
    ax12 = plt_pointing_hist(4,8,13, fig, int_dict=tile_integration, tile=tiles[14])
    ax13 = plt_pointing_hist(4,8,14, fig, int_dict=tile_integration, tile=tiles[15])
    ax14 = plt_pointing_hist(4,8,15, fig, int_dict=tile_integration, tile=tiles[16])
    ax15 = plt_pointing_hist(4,8,16, fig, int_dict=tile_integration, tile=tiles[17])
    ax16 = plt_pointing_hist(4,8,17, fig, int_dict=tile_integration, tile=tiles[2])
    ax17 = plt_pointing_hist(4,8,18, fig, int_dict=tile_integration, tile=tiles[18])
    ax18 = plt_pointing_hist(4,8,19, fig, int_dict=tile_integration, tile=tiles[19])
    ax19 = plt_pointing_hist(4,8,20, fig, int_dict=tile_integration, tile=tiles[20])
    ax20 = plt_pointing_hist(4,8,21, fig, int_dict=tile_integration, tile=tiles[21])
    ax21 = plt_pointing_hist(4,8,22, fig, int_dict=tile_integration, tile=tiles[22])
    ax22 = plt_pointing_hist(4,8,23, fig, int_dict=tile_integration, tile=tiles[23])
    ax23 = plt_pointing_hist(4,8,24, fig, int_dict=tile_integration, tile=tiles[24])
    ax24 = plt_pointing_hist(4,8,25, fig, int_dict=tile_integration, tile=tiles[3])
    ax25 = plt_pointing_hist(4,8,26, fig, int_dict=tile_integration, tile=tiles[25])
    ax26 = plt_pointing_hist(4,8,27, fig, int_dict=tile_integration, tile=tiles[26])
    ax27 = plt_pointing_hist(4,8,28, fig, int_dict=tile_integration, tile=tiles[27])
    ax28 = plt_pointing_hist(4,8,29, fig, int_dict=tile_integration, tile=tiles[28])
    ax29 = plt_pointing_hist(4,8,30, fig, int_dict=tile_integration, tile=tiles[29])
    ax30 = plt_pointing_hist(4,8,31, fig, int_dict=tile_integration, tile=tiles[30])
    ax31 = plt_pointing_hist(4,8,32, fig, int_dict=tile_integration, tile=tiles[31])



#    plt.title('Pointing Integration per Tile')
#    plt.ylabel('Hours')
#    plt.xlabel('Pointing [0, 2, 4]')
#    fig.text(0.5, 0.04, 'MWA Receiver Pointing [0, 2, 4]', ha='center')
#    fig.text(0.04, 0.5, 'Hours', va='center', rotation='vertical')
    plt.tight_layout()
    fig.savefig(f'{out_dir}/tiles_pointing_integration.png')









import os
import sys
import json
import argparse
from itertools import repeat
import sat_ephemeris as se
import sat_ids

import numpy as np
import skyfield as sf
from pathlib import Path
import concurrent.futures
from astropy.time import Time
from skyfield.api import Topos, load


# Save ephem date for one satellite
def sat_json(tle_dir, cadence, out_dir, sat_id):
    try:
        tle_path = f'{tle_dir}/{sat_id}.txt'
        
        sat_ephem = {}
        sat_ephem['sat_id'] = [sat_id]
        sat_ephem['time_array'] = []
        sat_ephem['sat_alt'] = []
        sat_ephem['sat_az'] = []
        
        sats, epochs = se.load_tle(tle_path)
        epoch_range = se.epoch_ranges(epochs)
        
        for i in range(len(epoch_range) - 1):
            try:
                t_arr, index_epoch = se.epoch_time_array(epoch_range, i, cadence)
                passes, alt, az = se.sat_pass(sats, t_arr, index_epoch) 
                
                for pass_index in passes:
                    time_array, sat_alt, sat_az = se.ephem_data(t_arr, pass_index, alt, az)
       
                    time_array = list(time_array)

                    # We don't care about sat passes shorter than a minute (3*20sec)
                    if len(time_array) >= 3:
                        sat_ephem['time_array'].append(time_array)
                        sat_ephem['sat_alt'].append(sat_alt)
                        sat_ephem['sat_az'].append(sat_az)
            except Exception:
                pass
        with open(f'{out_dir}/{sat_id}.json', 'w') as outfile:
            json.dump(sat_ephem, outfile, indent=4) 
        
        return f'Saved {sat_id}.json'
    except Exception:
        return f'ERROR! Couldn\'t save {sat_id}.json. TLE was empty!'


if __name__ == "__main__":


    # Skyfield Timescale
    ts = load.timescale(builtin=True)
    
    # Position of MWA site in Lat/Lon/Elevation
    MWA = Topos(latitude=-26.703319, longitude=116.670815, elevation_m=337.83)
    
    parser = argparse.ArgumentParser(description="""
            Code which converts the TLE files downloaded with download_TLE.py
            into satellite ephemeris data: rise time, set time, alt/az arrays
            at a given time cadence. This is saved to a json file which will 
            be used to plot the satellite passes.
            """)
            
    parser.add_argument('--tle_dir', metavar='\b', default='./../../outputs/sat_ephemeris/TLE', help='Directory where TLE files are saved. Default=./../../outputs/sat_ephemeris/TLE')
    parser.add_argument('--cadence', metavar='\b', type=int,  default=4, help='Rate at which sat alt/az is computed. Default=4s')
    parser.add_argument('--out_dir', metavar='\b', default='./../../outputs/sat_ephemeris/ephem_json/', help='Path to output directory. Default=./../../outputs/sat_ephemeris/ephem_json/')
    
    args = parser.parse_args()
    
    tle_dir = args.tle_dir
    cadence = int(args.cadence)
    out_dir = args.out_dir
    
    # Load list of satellites (Nodar ids)
    sat_list = list(sat_ids.norad_ids.values())
    
    # Save logs 
    Path(out_dir).mkdir(parents=True, exist_ok=True)
    sys.stdout = open(f'{out_dir}/sat_ephem_logs.txt', 'a')
    
    # Parellization Magic Here!
    with concurrent.futures.ProcessPoolExecutor() as executor:
        results = executor.map(
                sat_json, 
                repeat(tle_dir),
                repeat(cadence),
                repeat(out_dir),
                sat_list)
    
    for result in results:
        print(result)
    



import os
import sys
import rf_data as rf
from pathlib import Path
import concurrent.futures
from itertools import repeat
from datetime import datetime, timedelta


def plt_single_waterfall(rf_dir, rf_filename, out_dir):
    """Plot a single waterfall"""

    power, times = rf.read_data(f'{rf_dir}/{rf_filename}.txt')
    plt = rf.plot_waterfall(power, times, rf_filename)
    
    # Make output dir if it doesn't exist
    os.makedirs(os.path.dirname(out_dir), exist_ok=True)
    
    plt.savefig(f'{out_dir}/{rf_filename}.png')
    plt.close()


def waterfall_plot(tile, out_dir, data_dir, date, time_stamp):
    rf_name = f'{tile}_{time_stamp}'
    rf_path = Path(f'{data_dir}/{tile}/{date}')
    
    try:
        power, times = rf.read_data(f'{rf_path}/{rf_name}.txt')

        plt = rf.plot_waterfall(power, times, rf_name)

        save_dir = Path(f'{out_dir}/waterfalls/{date}/{time_stamp}')
        save_dir.mkdir(parents=True, exist_ok=True)
        
        
        plt.savefig(f'{save_dir}/{rf_name}.png')
        plt.close()
        return f'Waterfall {rf_name}.png saved'
    except Exception:
        return f'File {rf_name}.txt missing'

if __name__ == "__main__":

    import argparse
    parser = argparse.ArgumentParser(description="""
        Plots waterfall plots for all data 
        between two date ranges. Plot for all
        tiles. Saves plots in organised directory
        structure.
        """)
    
    parser.add_argument('--data_dir', metavar='\b', help='Dir where date is saved')
    parser.add_argument('--start_date', metavar='\b', help='Date from which to start plotting waterfalls. Ex: 2019-10-10')
    parser.add_argument('--stop_date', metavar='\b', help='Date until which to plot waterfalls. Ex: 2019-10-11')
    parser.add_argument('--out_dir', metavar='\b', default='./../../outputs/decode_rf_data',help='Output directory. Default=./../../outputs/decode_rf_data/')
    
    args = parser.parse_args()
    data_dir = args.data_dir
    start_date = args.start_date
    stop_date = args.stop_date
    out_dir = args.out_dir
    
    # Save logs 
    Path(out_dir).mkdir(parents=True, exist_ok=True)
    sys.stdout = open(f'{out_dir}/logs_{start_date}_{stop_date}.txt', 'a')
    
    # Import list of tile names from rf_data.py
    tiles = rf.tile_names()
    
    
    t_start = datetime.strptime(start_date, '%Y-%m-%d')
    t_stop = datetime.strptime(stop_date, '%Y-%m-%d')
    n_days = (t_stop - t_start).days
    
    dates = []
    date_time = []
    
    for i in range(n_days+1):
        day = t_start + timedelta(days=i)
        date = day.strftime('%Y-%m-%d')
        dates.append(date)
        d_t = []
        
        for j in range(48):
            t_delta = datetime.strptime(date,'%Y-%m-%d') + timedelta(minutes=30*j)
            d_time = t_delta.strftime('%Y-%m-%d-%H:%M')
            d_t.append(d_time)
    
        date_time.append(d_t)    
    
    
    data_dir = Path(data_dir)
    out_dir = Path(out_dir)
    
    for tile in tiles:
        for d in range(len(dates)):
            
            with concurrent.futures.ProcessPoolExecutor() as executor:
                results = executor.map(
                        waterfall_plot,
                        repeat(tile),
                        repeat(out_dir), 
                        repeat(data_dir), 
                        repeat(dates[d]), 
                        date_time[d])
               
            for result in results:
                print(result)






import json
import numpy as np
from pathlib import Path
from astropy.io import fits
import matplotlib.pylab as pl
import matplotlib.pyplot as plt


def find_flag(metafits, tiles):

    # dictionary to save flag info.
    # 0 indicates no dead dipole
    # 1-16 dipole index
    flags = {}
    flags['obsid'] = []
    for t in tiles:
        for p in ['X', 'Y']:
            flags[f'{t}{p}'] = []

    meta_files = [item for item in metafits.glob('*.metafits')]
    for m in meta_files: 

        hdu = fits.open(m)
        mode = hdu[0].header['MODE']
        obsid = hdu[0].header['GPSTIME']
        delays = hdu[1].data['Delays']
               
        tile_names = hdu[1].data['TileName']
        delays = hdu[1].data['Delays']
        pols = hdu[1].data['Pol']
        
        if mode == 'HW_LFILES':

            flags['obsid'].append(int(obsid))
            
            for t in tiles:
                idx = np.where(tile_names == t)
                for i in range(2):
                    idx_p = idx[0][i]
                    
                    t_name = tile_names[idx_p]
                    t_pol = pols[idx_p]
                    t_del = delays[idx_p]
                    t_flag = np.where(t_del == 32)[0] + 1
                     
                    if t_flag.size != 0:
                        flags[f'{t_name}{t_pol}'].append(int(t_flag[0]))
                    else:
                        flags[f'{t_name}{t_pol}'].append(0)

        hdu.close()

    return flags



if __name__ == "__main__":

    import argparse
    parser = argparse.ArgumentParser(description="""
            Look for dead dipoles
            """)
    
    parser.add_argument(
            '--metafits', metavar='\b', default='./../../outputs/beam_pointings/metafits/', 
            help='Directory where metafits files live. Default=./../../outputs/beam_pointings/metafits/')
    
    parser.add_argument(
            '--out_dir', metavar='\b', default='./../../outputs/beam_pointings/', 
            help='Directory where metafits files live. Default=./../../outputs/beam_pointings/')
    
    args = parser.parse_args()
    metafits     = Path(args.metafits)
    out_dir       = Path(args.out_dir)
    
    
    # list of all tiles used in this experiment
    tiles = [
            'HexS6', 'HexS7',
            'HexS8', 'HexS9',
            'HexS10', 'HexS12',
            'HexS29', 'HexS30',
            'HexS31', 'HexS32',
            'HexS33', 'HexS34', 
            'HexS35', 'HexS36']

    flags = find_flag(metafits, tiles)

    #for m in meta_files:
    #    find_flag(m)
    
    # Save flagging info to json file
    with open(f'{out_dir}/flags.json', 'w') as outfile:
        json.dump(flags, outfile, indent=4)
    
    keys = list(flags.keys())
    
    n = len(keys) - 1
    colors = pl.cm.Spectral(np.linspace(0,1,n))
    
    plt.style.use('seaborn')
    
    fig, axs = plt.subplots(4,7, figsize=(18, 9), sharex=True, sharey=True,)
    axs = axs.ravel()
    
    for i in range(n):
        axs[i].scatter(flags['obsid'], flags[keys[i+1]], color=colors[i], linewidths=0.1, s=28 ,alpha=0.88, label=keys[i+1])
        axs[i].set_ylim(-1, 17)
        axs[i].set_yticks([0, 4, 8, 12, 16])
        
        # these are matplotlib.patch.Patch properties
        props = dict(boxstyle='round', facecolor='wheat', alpha=0.2)
    
        # place a text box in upper left in axes coords
        axs[i].text(0.67, 0.93, f'{keys[i+1]}', transform=axs[i].transAxes, fontsize=10,
            verticalalignment='top', bbox=props)
    
    plt.tight_layout()
    plt.savefig(f'{out_dir}/dead_dipoles.png')
    
    
    



import sys
import json
import argparse
import matplotlib
import numpy as np
import healpy as hp
matplotlib.use('Agg')
from pathlib import Path
import concurrent.futures
import scipy.optimize as opt
from null_test import rotate
import matplotlib.pyplot as plt
from scipy.stats import chisquare
from scipy.stats import median_absolute_deviation as mad

sys.path.append('../sat_ephemeris')
from sat_ids import norad_ids

sys.path.append('../sat_channels')
from sat_channels import time_tree, time_filter

# Custom spectral colormap
sys.path.append('../decode_rf_data')
from rf_data import tile_names
from colormap import spectral
cmap = spectral()


def check_pointing(timestamp, point_0, point_2, point_4, point_41):
    '''Check if timestamp is at pointing 0, 2, 4, 41'''
    if timestamp in point_0:
        point = 0
    elif timestamp in point_2:
        point = 2
    elif timestamp in point_4:
        point = 4
    else:
        point=41
    
    return point

def read_aligned(ali_file=None):
    '''Read aligned data npz file'''
    paired_data = np.load(ali_file, allow_pickle=True)
    
    ref_p   = paired_data['ref_p_aligned']
    tile_p  = paired_data['tile_p_aligned']
    times   = paired_data['time_array']

    return [ref_p, tile_p, times]


def noise_floor(sat_thresh, noi_thresh, data=None):
    '''Computes the noise floor of the data '''
    
    # compute the standard deviation of data, and use it to identify occupied channels
    σ = np.std(data)
    
    # Any channel with a max power >= σ has a satellite
    sat_cut = sat_thresh*σ
    chans_pow_max = np.amax(data, axis=0)
    
    # Exclude the channels with sats, to only have noise data
    noise_chans = np.where(chans_pow_max < sat_cut)[0]
    noise_data = data[:, noise_chans]
    
    # noise median, noise mad, noise threshold = μ + 3*σ
    μ_noise = np.median(noise_data)
    σ_noise = mad(noise_data, axis=None)
    noise_threshold = μ_noise + noi_thresh*σ_noise
    
    return (data, noise_threshold)


# chisquared minimization to best fit map to data
def fit_gain(map_data=None,fee=None):
    '''Fit the beam model to the measured data using
    chisquared minimization'''
    
    bad_values = np.isnan(map_data)
    map_data = map_data[~bad_values]
    fee = fee[~bad_values]
    
    def chisqfunc(gain):
        model = fee + gain
        chisq = sum((map_data - model)**2)
        return chisq

    x0 = np.array([0])

    result =  opt.minimize(chisqfunc,x0)
    
    return result.x


def fit_test(map_data=None,fee=None):
    '''chi-squared test for fit between
    model and data'''
    
    bad_values = np.isnan(map_data)
    map_data = map_data[~bad_values]
    fee = fee[~bad_values]
    
    #map_data = np.asarray(map_data) + 120
    #fee = np.asarray(fee) + 120
    map_data = np.asarray(map_data) - np.nanmin(fee) + 20
    fee = np.asarray(fee) - np.nanmin(fee) + 20

   
    _, pvalue = chisquare(map_data, f_exp=fee)

    return pvalue


def plt_channel(
        out_dir, times, ref_c, tile_c,
        ref_noise, tile_noise, chan_num,
        sat_id, pointing, date, point):

    '''Plot power in channel, with various thresholds
    
    Args:
        times:          Time array
        channel_power:  Power in channel
        chan_num:       Channel Number
        min_s:          Minimum signal in channel_power
        max_s:          Maximum signal in channel_power
        noise_threshold: Noise Threshold (n*MAD)
        sat_id:         Norad Cat ID
        date:           Date of observation
        '''
    
    
    plt.style.use('seaborn')

    ylims = [min(ref_c), max(ref_c), min(tile_c), max(tile_c)]
    min_s = min(ylims)
    max_s = max(ylims)
   
    fig = plt.figure(figsize=(8,6))
    fig.suptitle(f'Satellite [{sat_id}] Pass @ {date} in Channel: [{chan_num}]', y=0.98)

    ax1 = fig.add_subplot(2, 1, 1)
    ax1.plot(times, ref_c, linestyle='-', linewidth=2, alpha=1.0, color='#729d39', label='ref')
    ax1.fill_between(times, y1=ref_c, y2=-120, color='#729d39', alpha=0.7)
    ax1.axhline(ref_noise,linestyle='-', linewidth=2, color='#36622b', label=f'Ref Cut: {ref_noise:.2f} dBm')
    ax1.axhspan(-120, ref_noise, color='#36622b', alpha=0.7)
    ax1.set_ylim([min(ref_c)-1, max(ref_c)+1])
    ax1.set_ylabel('Power [dBm]')
    ax1.set_xticklabels([])
    
    leg = ax1.legend(frameon=True)
    leg.get_frame().set_facecolor('grey')
    leg.get_frame().set_alpha(0.2)
    for l in leg.legendHandles:
        l.set_alpha(1)
    
    
    ax2 = fig.add_subplot(2, 1, 2)
    ax2.plot(times, tile_c, linestyle='-', linewidth=2, alpha=1.0, color='#ff5656', label='tile')
    ax2.fill_between(times, y1=tile_c, y2=-120, color='#ff5656', alpha=0.7)
    ax2.axhline(tile_noise,linestyle='-', linewidth=2, color='#970747', label=f'Tile Cut: {tile_noise:.2f} dBm')
    ax2.axhspan(-120, tile_noise, color='#970747', alpha=0.7)
    ax2.set_ylim([min(tile_c)-1, max(tile_c)+1])
    ax2.set_ylabel('Power [dBm]')
    ax2.set_xlabel('Time [s]')
    
    leg = ax2.legend(frameon=True)
    leg.get_frame().set_facecolor('grey')
    leg.get_frame().set_alpha(0.2)
    for l in leg.legendHandles:
        l.set_alpha(1)
    
    plt.tight_layout()
    plt.subplots_adjust(top=0.94)
    plt.savefig(f'{out_dir}/{date}_{sat_id}_{chan_num}_channel.png')
    plt.close()


def plt_fee_fit(t, mwa_fee_pass, mwa_pass_fit, mwa_pass_fit_raw, out_dir, point, timestamp, sat):

    pval = fit_test(map_data=mwa_pass_fit, fee=mwa_fee_pass)

    plt.style.use('seaborn')

    fig = plt.figure(figsize=(8,6))
    ax1 = fig.add_subplot(1, 1, 1)
    

    ax1.scatter(t, mwa_fee_pass, color='#c70039', alpha=0.6, marker=".", label="fee_slice")

    where_nan = np.isnan(mwa_pass_fit)
    
    ax1.scatter(t, mwa_pass_fit_raw, color='#7da87b', alpha=0.9, marker=".", label="RFE raw")
    ax1.set_ylim(-50, 2)
    
    ax1.scatter(t, mwa_pass_fit, color='#4f8a8b', alpha=0.9, marker=".", label="RFE cali")
    ax1.set_ylim(-50, 2)
    
    leg = ax1.legend(frameon=True)
    leg.get_frame().set_facecolor('grey')
    leg.get_frame().set_alpha(0.2)
    for l in leg.legendHandles:
        l.set_alpha(1)
    
    plt.title(f'Goodness of Fit: {pval} ')
    plt.tight_layout()
    plt.savefig(f'{out_dir}/{point}/{timestamp}_{sat}.png')
    plt.close()


def power_ephem(
        ref, tile,
        ali_file,
        chrono_file,
        sat_id,
        sat_chan,
        point,
        pow_thresh,
        timestamp
        ):

    '''Create power, alt, az arrays at constant cadence'''

    # Read .npz aligned file
    ref_p, tile_p, times = read_aligned(ali_file=ali_file)

    # Scale noise floor to zero and determine noise threshold
    ref_p, ref_noise = noise_floor(sat_thresh, noi_thresh, ref_p)
    tile_p, tile_noise = noise_floor(sat_thresh, noi_thresh, tile_p)
    
    with open(chrono_file) as chrono:
        chrono_ephem = json.load(chrono)
    
        norad_list = [chrono_ephem[s]["sat_id"][0] for s in range(len(chrono_ephem))]
        
        norad_index = norad_list.index(sat_id)
        
        norad_ephem = chrono_ephem[norad_index]
            
        rise_ephem  = norad_ephem["time_array"][0] 
        set_ephem   = norad_ephem["time_array"][-1]
        
        intvl = time_filter(rise_ephem, set_ephem, np.asarray(times))
        
        if intvl != None:
            
            w_start, w_stop = intvl

            # Slice [crop] the ref/tile/times arrays to the times of sat pass and extract sat_chan
            ref_c = ref_p[w_start:w_stop+1, sat_chan]
            tile_c = tile_p[w_start:w_stop+1, sat_chan]
            times_c = times[w_start:w_stop+1]
            
            alt = np.asarray(norad_ephem["sat_alt"])
            az  = np.asarray(norad_ephem["sat_az"])

            if ((np.nanmax(ref_c - ref_noise) >= pow_thresh) and (np.nanmax(tile_c - tile_noise) >= pow_thresh)):

                # Apply noise criteria. In the window, where are ref_power and tile power
                # above their respective thresholds?
                if np.where((ref_c >= ref_noise) & (tile_c >= tile_noise))[0].size != 0: 
                    good_ref    = ref_c[np.where((ref_c >= ref_noise) & (tile_c >= tile_noise))[0]]
                    good_tile   = tile_c[np.where((ref_c >= ref_noise) & (tile_c >= tile_noise))[0]]
                    good_alt    = alt[np.where((ref_c >= ref_noise) & (tile_c >= tile_noise))[0]]
                    good_az     = az[np.where((ref_c >= ref_noise) & (tile_c >= tile_noise))[0]]

                    if plots == 'True':
                        plt_channel(f'{out_dir}/pass_plots/{tile}_{ref}/{point}', times_c, ref_c, tile_c, ref_noise, tile_noise, sat_chan, sat_id, point, timestamp, point)
                    
                    return [good_ref, good_tile, good_alt, good_az, times_c]

                else:
                    return 0
            
            else:
                return 0

        else:
            return 0


def project_tile_healpix(tile_pair):

    ref, tile = tile_pair

    pointings = ['0','2','4', '41']
    
    if plots == 'True':
        Path(f'{out_dir}/pass_plots/{tile}_{ref}/0').mkdir(parents=True, exist_ok=True)
        Path(f'{out_dir}/pass_plots/{tile}_{ref}/2').mkdir(parents=True, exist_ok=True)
        Path(f'{out_dir}/pass_plots/{tile}_{ref}/4').mkdir(parents=True, exist_ok=True)
        Path(f'{out_dir}/pass_plots/{tile}_{ref}/41').mkdir(parents=True, exist_ok=True)
        
        Path(f'{out_dir}/fit_plots/{tile}_{ref}/0').mkdir(parents=True, exist_ok=True)
        Path(f'{out_dir}/fit_plots/{tile}_{ref}/2').mkdir(parents=True, exist_ok=True)
        Path(f'{out_dir}/fit_plots/{tile}_{ref}/4').mkdir(parents=True, exist_ok=True)
        Path(f'{out_dir}/fit_plots/{tile}_{ref}/41').mkdir(parents=True, exist_ok=True)

    # Initialize an empty dictionary for tile data
    # The map is list of length 12288 of empty lists to append pixel values to
    # keep track of which satellites contributed which data
    tile_data = {
            'mwa_maps':{p:[[] for pixel in range(hp.nside2npix(nside))] for p in pointings}, 
            'ref_maps':{p:[[] for pixel in range(hp.nside2npix(nside))] for p in pointings}, 
            'tile_maps':{p:[[] for pixel in range(hp.nside2npix(nside))] for p in pointings}, 
            'sat_map':{p:[[] for pixel in range(hp.nside2npix(nside))] for p in pointings},
            'times':{p:[[] for pixel in range(hp.nside2npix(nside))] for p in pointings}
            }
    
    
    # Load reference FEE model
    # Rotate the fee models by -pi/2 to move model from spherical (E=0) to Alt/Az (N=0)
    ref_fee_model = np.load(ref_model, allow_pickle=True)
    
#    # Tile S33YY has it's 9th dipole flagged
#    if tile is 'S33YY':
#        fee_m = np.load(fee_map_flagged, allow_pickle=True)
#    else:
#        fee_m = np.load(fee_map, allow_pickle=True)
    fee_m = np.load(fee_map, allow_pickle=True)
        
    if 'XX' in tile:    
        ref_fee = ref_fee_model['XX']
        rotated_fee = rotate(nside, angle=-(1*np.pi)/2.0,healpix_array=ref_fee)
    else:
        ref_fee = ref_fee_model['YY']
        rotated_fee = rotate(nside, angle=-(1*np.pi)/2.0,healpix_array=ref_fee)
    
    for day in range(len(dates)):

        for window in range(len(date_time[day])):
            timestamp = date_time[day][window]
            
            # Check if at timestamp, reciever was pointed to 0,2,4 gridpointing
            if ((timestamp in point_0) or (timestamp in point_2) or (timestamp in point_4) or (timestamp in point_41)):
            
                # pointing at timestamp
                point = check_pointing(timestamp, point_0, point_2, point_4, point_41)

                if 'XX' in tile:    
                    mwa_fee = fee_m[str(point)][0]
                else:
                    mwa_fee = fee_m[str(point)][1]

                ali_file = Path(f'{align_dir}/{dates[day]}/{timestamp}/{ref}_{tile}_{timestamp}_aligned.npz')

                # check if file exists
                if ali_file.is_file():

                    # Chrono and map Ephemeris file
                    chrono_file = Path(f'{chrono_dir}/{timestamp}.json')
                    channel_map = f'{map_dir}/{timestamp}.json'

                    with open(chrono_file) as chrono:
                        chrono_ephem = json.load(chrono)
                    
                        if chrono_ephem != []:
                
                            norad_list = [chrono_ephem[s]["sat_id"][0] for s in range(len(chrono_ephem))]

                            if norad_list != []:

                                with open(channel_map) as ch_map:
                                    chan_map = json.load(ch_map)
                                
                                    chan_sat_ids = [int(i) for i in list(chan_map.keys())]
                        
                                    for sat in chan_sat_ids:

                                        chan = chan_map[f'{sat}']

                                        sat_data = power_ephem(
                                                ref, tile,
                                                ali_file,
                                                chrono_file,
                                                sat,
                                                chan,
                                                point,
                                                pow_thresh,
                                                timestamp
                                                )

                                        if sat_data != 0:
                                        
                                            ref_power, tile_power, alt, az, times = sat_data


                                            # Altitude is in deg while az is in radians
                                            # convert alt to radians
                                            # za - zenith angle
                                            alt = np.radians(alt)
                                            za = np.pi/2 - alt
                                            az  = np.asarray(az)


                                            # Now convert to healpix coordinates
                                            #healpix_index = hp.ang2pix(nside,θ, ɸ)
                                            healpix_index = hp.ang2pix(nside, za, az)
                                            
                                            # multiple data points fall within a single healpix pixel
                                            # find the unique pixels
                                            u = np.unique(healpix_index)

                                            ref_pass = np.array([np.nanmean(ref_power[np.where(healpix_index==i)[0]]) for i in u])
                                            tile_pass = np.array([np.nanmean(tile_power[np.where(healpix_index==i)[0]]) for i in u])
                                            times_pass = np.array([np.mean(times[np.where(healpix_index==i)][0]) for i in u])
                                            ref_fee_pass = np.array([rotated_fee[i] for i in u])
                                            mwa_fee_pass = np.array([mwa_fee[i] for i in u])

                                            # implement RFE gain corrections here
                                            rfe_polyfit = np.load(rfe_gain)
                                            gain_cal = np.poly1d(rfe_polyfit)
                                            #rfe_thresh = gain_cal.roots[0]
                                            rfe_thresh = max(gain_cal.roots)
                                            tile_pass_rfe = [i+gain_cal(i) if i >= rfe_thresh else i for i in tile_pass]
                                            

                                            # magic here
                                            # the beam shape finally emerges
                                            #mwa_pass = np.array(tile_pass) - np.array(ref_pass) + np.array(ref_fee_pass)
                                            mwa_pass = np.array(tile_pass_rfe) - np.array(ref_pass) + np.array(ref_fee_pass)


                                            # fit the power level of the pass to the mwa_fee model using a single gain value
                                            offset = fit_gain(map_data=mwa_pass, fee=mwa_fee_pass)

                                            mwa_pass_fit = mwa_pass - offset[0]
                                            
                                            if mwa_pass_fit.size != 0:
                                                
                                                # determine how well the data fits the model with chi-square  
                                                pval = fit_test(map_data=mwa_pass_fit, fee=mwa_fee_pass)
                                                
                                                if plots == 'True':
                                                    mwa_pass_raw = np.array(tile_pass) - np.array(ref_pass) + np.array(ref_fee_pass)
                                                    offset = fit_gain(map_data=mwa_pass_raw, fee=mwa_fee_pass)
                                                    mwa_pass_fit_raw = mwa_pass_raw - offset[0]
                                                    
                                                    plt_fee_fit(times_pass, mwa_fee_pass, mwa_pass_fit, mwa_pass_fit_raw, f'{out_dir}/fit_plots/{tile}_{ref}/', point, timestamp, sat)
                                                
                                                # a goodness of fit threshold
                                                if pval >= 0.8:

                                                    # loop though all healpix pixels for the pass
                                                    for i in range(len(u)):

                                                        tile_data['mwa_maps'][f'{point}'][u[i]].append(mwa_pass_fit[i])
                                                        tile_data['ref_maps'][f'{point}'][u[i]].append(ref_pass[i])
                                                        tile_data['tile_maps'][f'{point}'][u[i]].append(tile_pass[i])
                                                        tile_data['times'][f'{point}'][u[i]].append(times_pass[i])
                                                        tile_data['sat_map'][f'{point}'][u[i]].append(sat)
                                           

                else:
                    print(f'Missing {ref}_{tile}_{timestamp}_aligned.npz')
                    continue

    # Sort data by satellites

    # list of all possible satellites
    sat_ids = list(norad_ids.values())
    
    # create a dictionary, with the keys being sat_ids and the values being healpix maps of data from those sats
    # Fist level keys are pointings with dictionaty values
    # Second level keys are satellite ids with healpix map lists
    mwa_sat_data = {}
    ref_sat_data = {}
    tile_sat_data = {}
    time_sat_data = {}

    # loop over pointings for all maps
    for p in pointings:
        
        mwa_map     = np.asarray(tile_data['mwa_maps'][p])
        tile_map    = np.asarray(tile_data['tile_maps'][p])
        ref_map     = np.asarray(tile_data['ref_maps'][p])
        sat_map     = np.asarray(tile_data['sat_map'][p])
        time_map    = np.asarray(tile_data['times'][p])

        mwa_sat_data[p]     = {}
        ref_sat_data[p]     = {}
        tile_sat_data[p]    = {}
        time_sat_data[p]    = {}
        
        # loop over all sats
        for s in sat_ids:

            mwa_sat_data[p][s]      = [] 
            ref_sat_data[p][s]      = [] 
            tile_sat_data[p][s]     = [] 
            time_sat_data[p][s]     = [] 
            

            # loop over every healpix pixel
            for i in range(len(ref_map)):

                # Find subset of data for each sat
                sat_idx = np.where(np.asarray(sat_map[i]) == s)
                mwa_sat_data[p][s].append((np.asarray(mwa_map[i])[sat_idx]).tolist())
                ref_sat_data[p][s].append((np.asarray(ref_map[i])[sat_idx]).tolist())
                tile_sat_data[p][s].append((np.asarray(tile_map[i])[sat_idx]).tolist())
                time_sat_data[p][s].append((np.asarray(time_map[i])[sat_idx]).tolist())

    # Save map arrays to npz file
    tile_sat_data = {'mwa_map':mwa_sat_data, 'ref_map':ref_sat_data, 'tile_map':tile_sat_data, 'time_map':time_sat_data}
    np.savez_compressed(f'{out_dir}/{tile}_{ref}_sat_maps.npz', **tile_sat_data)


if __name__=='__main__':

    parser = argparse.ArgumentParser(description="""
        Project a sat pass onto a healpix map using ephemeris data
        """)
    
    parser.add_argument(
            '--align_dir', metavar='\b',default='../../outputs/align_data/',
            help='Dir where agligned data date is saved. Default:../../outputs/align_data')

    parser.add_argument(
            '--out_dir', metavar='\b', default='./../../outputs/tile_maps/tile_maps_raw/',
            help='Output directory. Default=./../../outputs/tile_maps/tile_maps_raw/')
    
    parser.add_argument(
            '--obs_point', metavar='\b', default='../../outputs/beam_pointings/obs_pointings.json',
            help='Observation pointing lists. Default=../../outputs/beam_pointings/obs_pointings.json')

    parser.add_argument(
            '--chrono_dir', metavar='\b', default='./../../outputs/sat_ephemeris/chrono_json',
            help='Output directory. Default=./../../outputs/sat_ephemeris/chrono_json/')
    
    parser.add_argument(
            '--map_dir', metavar='\b', default='../../outputs/sat_channels/window_maps/',
            help='Satellite channel map. Default=../../outputs/sat_channels/window_maps/')

    parser.add_argument('--ref_model', metavar='\b', default='../../outputs/reproject_ref/ref_dipole_models.npz',
            help='Healpix reference FEE model file. default=../../outputs/reproject_ref/ref_dipole_models.npz')
    
    parser.add_argument('--rfe_gain', metavar='\b', default='../../outputs/tile_maps/rfe_gain/rfe_gain_fit.npy',
            help='RF Explorer gain fit. default=../../outputs/tile_maps/rfe_gain/rfe_gain_fit.npy')
    
    parser.add_argument('--fee_map', metavar='\b', default='../../outputs/tile_maps/FEE_maps/mwa_fee_beam.npz',
            help='Healpix FEE map of mwa tile. default=../../outputs/tile_maps/FEE_maps/mwa_fee_beam.npz')
    
    parser.add_argument('--fee_map_flagged', metavar='\b', default='../../outputs/tile_maps/FEE_maps/mwa_fee_beam_9_flagged.npz',
            help='Healpix FEE map of mwa tile. default=../../outputs/tile_maps/FEE_maps/mwa_fee_beam_9_flagged.npz')
    
    parser.add_argument('--fit_thresh', metavar='\b', default=0.9, help='Goodness of fit threshold. 0.9 seems to only reject obvious outliers')
    parser.add_argument('--noi_thresh', metavar='\b', type=int, default=3,help='Noise Threshold: Multiples of MAD. Default=3.')
    parser.add_argument('--sat_thresh', metavar='\b', type=int, default=1,help='σ threshold to detect sats Default=1.')
    parser.add_argument('--pow_thresh', metavar='\b', type=int, default=5,help='Power Threshold to detect sats. Default=10 dB.')
    parser.add_argument('--nside', metavar='\b', type=int,  default=32,help='Healpix Nside. Default = 32')
    parser.add_argument('--plots', metavar='\b', default=False,help='If True, create a gazzillion plots for each sat pass. Default = False')
    parser.add_argument('--start_date', metavar='\b', help='Date from which to start aligning data. Ex: 2019-10-10')
    parser.add_argument('--stop_date', metavar='\b', help='Date until which to align data. Ex: 2019-10-11')
    
    args = parser.parse_args()
    
    start_date      = args.start_date
    stop_date       = args.stop_date
    obs_point       = args.obs_point
    noi_thresh      = args.noi_thresh
    sat_thresh      = args.sat_thresh
    pow_thresh      = args.pow_thresh
    fit_thresh      = args.fit_thresh
    nside           = args.nside
    plots           = args.plots
    ref_model       = args.ref_model
    rfe_gain        = args.rfe_gain
    fee_map         = args.fee_map
    fee_map_flagged = args.fee_map_flagged
    
    align_dir       = Path(args.align_dir)
    chrono_dir      = Path(args.chrono_dir)
    out_dir         = Path(args.out_dir)
    map_dir         = Path(args.map_dir)

    # Tile names
    refs    = tile_names()[:4]
    tiles   = tile_names()[4:]
    
    # All relevant tile pairs
    tile_pairs = []
    for ref in refs:
        if 'XX' in ref:
            for tile in [t for t in tiles if 'XX' in t]:
                tile_pairs.append([ref, tile])
        else:
            for tile in [t for t in tiles if 'YY' in t]:
                tile_pairs.append([ref,tile])

    # Read observation pointing list
    with open(obs_point) as point:
        obs_p = json.load(point)
        point_0  = obs_p['point_0'] 
        point_2  = obs_p['point_2'] 
        point_4  = obs_p['point_4']
        point_41 = obs_p['point_41']


    # dates: list of days
    # date_time = list of 30 min observation windows
    dates, date_time = time_tree(start_date, stop_date)

    # Save logs 
    Path(out_dir).mkdir(parents=True, exist_ok=True)
    sys.stdout = open(f'{out_dir}/logs_{start_date}_{stop_date}.txt', 'a')
   
#    project_tile_healpix(tile_pairs[0])
#    project_tile_healpix(['rf0YY', 'S33YY'])
        
    # Parallization magic happens here
    with concurrent.futures.ProcessPoolExecutor() as executor:
        results = executor.map(project_tile_healpix, tile_pairs)





import json
import matplotlib
import numpy as np
matplotlib.use('Agg')
import seaborn as sns
import matplotlib.pyplot as plt
    


def point_integration(f_name, out_dir, int_thresh):
    """Bin data to calculate integration at each pointing"""
    
    with open(f'{out_dir}/{f_name}', 'r') as data:
        pointings = json.load(data)
        grid_pt = pointings['grid_pt']
        obs_length = pointings['obs_length']
    
    # find unique pointings. 
    unique_pointings = list(set(grid_pt))
    unique_pointings = sorted(unique_pointings)
    
    
    pointings = []
    integrations = []
    
    for i in unique_pointings:
        pointings.append(i)
        time = 0
        for j in range(len(grid_pt)):
            if i == grid_pt[j]:
                time = time + obs_length[j]
        integrations.append(time)       
    
    
    int_hours = np.asarray(integrations)/(60*60)
    
    time_point = []
    point = []
    
    # time threshold at pointing in hours
    point_threshold = int_thresh
    
    for i in range(len(int_hours)):
        if int_hours[i] >= point_threshold:
            point.append(pointings[i])
            time_point.append(int_hours[i])

    return (time_point, point)


def pointing_hist(time_point, point, out_dir):
    """Pointing histogram"""

    x = range(len(time_point))
    leg = [int(i) for i in time_point]
    
    plt.style.use('seaborn')
    fig, ax = plt.subplots(figsize=(8,6))
    pal = sns.cubehelix_palette(len(time_point), start=.4, rot=-.5, dark=.4, reverse=True)
    barplot = plt.bar(x, time_point, color=sns.color_palette(pal))
    
    def autolabel(rects):
        for idx,rect in enumerate(barplot):
            height = rect.get_height()
            ax.text(rect.get_x() + rect.get_width()/2., height,
                    leg[idx],
                    ha='center', va='bottom', rotation=0)
    
    autolabel(barplot)
    
    plt.xticks(x, point)
    plt.ylabel('Hours')
    plt.xlim(-0.7,len(time_point)-0.3)
    plt.xlabel('MWA Grid Pointing Number')
    plt.title('Integration at MWA Grid Pointings')
    plt.tight_layout()
    plt.savefig(f'{out_dir}/pointing_integration.png')


if __name__ == "__main__":

    import argparse

    parser = argparse.ArgumentParser(description="""
            Reads ultimate_pointing_times.json, and plots
            a histogram of total integration time at various
            pointings.
            """)
    
    parser.add_argument('--out_dir', metavar='\b', default='./../../outputs/beam_pointings/', help='Directory where json metadata lives. Default=./../../outputs/beam_pointings/')
    parser.add_argument('--f_name', metavar='\b', default='ultimate_pointing_times.json', help='File name of json to be plotted. Default=ultimate_pointing_times.json')
    parser.add_argument('--int_thresh', metavar='\b', default=200, type=int,  help='Pointing integration time threshold. Default=200')
    
    args = parser.parse_args()
    out_dir     = args.out_dir
    f_name      = args.f_name
    int_thresh  = args.int_thresh

    time_point, point = point_integration(f_name, out_dir, int_thresh)
    pointing_hist(time_point, point, out_dir)




from astropy.time import Time

def convert_gps(date):
    return int(Time(date, format='iso').gps)

if __name__ == '__main__':
    import argparse
    parser = argparse.ArgumentParser(description="""
            Convert date-time [iso] to gps time
            """)
    parser.add_argument('--date', metavar='\b', help='Date-time to be converted to gps-seconds. Ex:2019-01-01 00:00:00')
    
    args = parser.parse_args()
    date = args.date
    
    gps = convert_gps(date)
    print(f'gps: {gps}')






import numpy as np
import healpy as hp
import scipy.optimize as opt
import numpy.polynomial.polynomial as poly
from scipy.stats import median_absolute_deviation as mad


def ring_indices(nside=None):
    '''Healpix pix indices of NS, EW slices and above horizon'''
    # theta phi values of each pixel 
    hp_indices = np.arange(hp.nside2npix(nside))
    θ, ɸ = hp.pix2ang(nside, hp_indices)

    za = np.arange(0,87,2)

    ring_indices = []
    for i in range(len(za)):
        if i < 43:
            r_indices = np.where(np.logical_and(θ >= np.radians(za[i]), θ < np.radians(za[i+1])))[0]
            ring_indices.append(r_indices)
    za_cen = za[:-1] + 1
    return [za_cen, ring_indices]

def good_maps(ref_map):
    '''Creates a ref map with only good satellites'''
   
    pointings = ['0','2','4']
    
    # load data from map .npz file
    f = Path(f'{map_dir}/{ref_map}')
    tile_data = np.load(f, allow_pickle=True)
    tile_data = {key:tile_data[key].item() for key in tile_data}
    ref_map = tile_data['ref_map']  
    
    # Good sats from which to make plots
    good_sats = [
            25338, 25982, 25984, 25985,
            28654, 40086, 40087,
            40091, 41179, 41180,
            41182, 41183, 41184,
            41185, 41187, 41188,
            41189, 44387
            ]
    
    orbcomm = [
            25982, 25984, 25985,
            40086, 40087, 40091,
            41179, 41180, 41182, 
            41183, 41184, 41185, 
            41187, 41188, 41189
            ]
    noaa = [25338, 28654]

    meteor = [44387]
    
    # Empty good map
    good_map = [[] for pixel in range(hp.nside2npix(nside))]
    
    for p in pointings:
        
        # append to good map from all good sat data
        for sat in meteor:
            for pix in range(hp.nside2npix(nside)):
                good_map[pix].extend(ref_map[p][sat][pix])
        
    mad_map = []
    for j in good_map:
        if j != []:
            j = np.asarray(j)
            j = j[~np.isnan(j)]
            mad_map.append(mad(j))
        else:
            mad_map.append(np.nan)
    
    good_map = [np.nanmedian(pixel) for pixel in good_map]
        
    return (good_map, mad_map)

def poly_fit(x, y, order):
    '''Fit polynominal of order to data'''
    
    x = np.asarray(x)
    y = np.asarray(y)

    coefs = poly.polyfit(x, y, order)
    fit = poly.polyval(x, coefs)
    return fit


if __name__=='__main__':
    
    import argparse
    import numpy as np
    from pathlib import Path
    import matplotlib
    matplotlib.use('Agg')
    import matplotlib.pyplot as plt
    import matplotlib.gridspec as gs
    from mpl_toolkits.axes_grid1 import make_axes_locatable
    from scipy.stats import median_absolute_deviation as mad

    import sys
    sys.path.append('../decode_rf_data')
    sys.path.append('../tile_maps')
    from plot_tile_maps import plot_healpix
    from colormap import spectral
    
    # Custom spectral colormap
    cmap = spectral()
    
    parser = argparse.ArgumentParser(description="""
        Plot healpix map of reference data
        """)
    
    parser.add_argument('--out_dir', metavar='\b', default='../../outputs/tile_maps/null_test/',
            help='Output directory. Default=../../outputs/tile_maps/null_test/')
    parser.add_argument('--map_dir', metavar='\b', default='../../outputs/tile_maps/tile_maps_raw/',
            help='Output directory. Default=../../outputs/tile_maps/tile_maps_raw/')
    parser.add_argument('--ref_model', metavar='\b', default='../../outputs/reproject_ref/ref_dipole_models.npz',
            help='Healpix reference FEE model file. default=../../outputs/reproject_ref/ref_dipole_models.npz')
    parser.add_argument('--nside', metavar='\b',type=int, default=32,help='Healpix Nside. Default = 32')
    
    
    args = parser.parse_args()
    
    out_dir     = Path(args.out_dir)
    map_dir     = Path(args.map_dir)
    ref_model   = args.ref_model
    nside       = args.nside
    
    out_dir.mkdir(parents=True, exist_ok=True)
    
    ref_tiles = [
            'S35XX_rf0XX_sat_maps.npz', 
            'S35YY_rf0YY_sat_maps.npz', 
            'S35XX_rf1XX_sat_maps.npz', 
            'S35YY_rf1YY_sat_maps.npz'
            ]

    
    za, indices = ring_indices(nside=nside)
    
    # median and mad values of induvidual ref maps 
    good_rf0XX, mad_rf0XX = good_maps(ref_tiles[0])
    good_rf0YY, mad_rf0YY = good_maps(ref_tiles[1])
    good_rf1XX, mad_rf1XX = good_maps(ref_tiles[2])
    good_rf1YY, mad_rf1YY = good_maps(ref_tiles[3])

    # Load reference FEE model
    ref_fee_model = np.load(ref_model, allow_pickle=True)
    beam_XX = ref_fee_model['XX']
    beam_YY = ref_fee_model['YY']
   
    # residuals of individual maps
    res_rf0XX = good_rf0XX - beam_XX
    res_rf0YY = good_rf0YY - beam_YY
    res_rf1XX = good_rf1XX - beam_XX
    res_rf1YY = good_rf1YY - beam_YY

    sum_array = np.array([res_rf0XX, res_rf0YY, res_rf1XX, res_rf1YY])
    mad_array = np.array([mad_rf0XX, mad_rf0YY, mad_rf1XX, mad_rf1YY])
    
    # sum along corresponding pixels
    res_median = np.nanmedian(sum_array, axis=0)
    mad_average = np.nanmean(mad_array, axis=0)
   
    # median of values in zenith angle rings
    res_rings = []
    mad_rings = []
    for i in indices:
        res_rings.append(np.nanmedian(res_median[i]))
        mad_rings.append(np.nanmedian(mad_average[i]))

    # Scale the residuals to 0 median 
    res_med = np.median(res_rings)
    res_rings = np.array(res_rings) - res_med

    fit_res = poly_fit(za, res_rings, 8)  
    
    plt.style.use('seaborn')
    nice_fonts = {
            # Use LaTeX to write all text
            #"text.usetex": True,
            "font.family": "sans-serif",
            # Use 10pt font in plots, to match 10pt font in document
            "axes.labelsize": 8,
            "font.size": 10,
            # Make the legend/label fonts a little smaller
            "legend.fontsize": 6,
            "xtick.labelsize": 8,
            "ytick.labelsize": 8,
            }
    
    plt.rcParams.update(nice_fonts)
    
    fig = plt.figure(figsize=(3.6,2.4))
           
    plt.style.use('seaborn')
    plt.errorbar(
           za, res_rings, yerr=mad_rings, 
           fmt='.', color='#326765', ecolor='#7da87b',
           elinewidth=1.4, capsize=1.4, capthick=1.6,
           alpha=0.9, ms=9, label=r'$\Delta$ref [$\theta$]')
    
    #plt.scatter(za, res_rings, marker='.', s=77, alpha=0.9,  color='#466844', label=r'$\Delta$ref [$\theta$]')
    plt.plot(za, fit_res, linewidth=1.4, alpha=1, color='#fa4659', label=r'8$^{th}$ order fit')
    
    leg = plt.legend(frameon=True, markerscale=1, handlelength=1)
    leg.get_frame().set_facecolor('white')
    for l in leg.legendHandles:
        l.set_alpha(1)
    
    plt.xlabel(r'Zenith Angle [degrees]')
    plt.ylabel('Residual Power [dB]')
    plt.tight_layout()
    #plt.savefig('../../outputs/paper_plots/ref_residuals.pdf', bbox_inches='tight')
    plt.savefig('ref_residuals_meteor.pdf', bbox_inches='tight')






import json
import argparse
import numpy as np
from pathlib import Path
from astropy.io import fits
import matplotlib.pylab as pl
import matplotlib.pyplot as plt

parser = argparse.ArgumentParser(description="""
        Look for dead dipoles
        """)

parser.add_argument(
        '--metafits', metavar='\b', default='./../../../outputs/beam_pointings/metafits/1262518040.metafits', 
        help='Directory where metafits files live. Default=./../../../outputs/beam_pointings/metafits/1262518040.metafits')

parser.add_argument(
        '--out_dir', metavar='\b', default='./../../../outputs/paper_plots/', 
        help='Directory where metafits files live. Default=./../../../outputs/paper_plots/')

args = parser.parse_args()
metafits     = Path(args.metafits)
out_dir       = Path(args.out_dir)

# list of all tiles used in this experiment
tiles_14 = [
        'HexS6', 'HexS7',
        'HexS8', 'HexS9',
        'HexS10', 'HexS12',
        'HexS29', 'HexS30',
        'HexS31', 'HexS32',
        'HexS33', 'HexS34', 
        'HexS35', 'HexS36'
        ]



# Read metafits file and extract positions of each tile in N, E coordinates
tiles = []
N = []
E = []
hdu = fits.open(metafits)
tile_names = hdu[1].data['TileName']
north = hdu[1].data['North']
east = hdu[1].data['East']
pols = hdu[1].data['Pol']

for t in range(len(tile_names)):
    if 'X' in pols[t]: 
        tiles.append(tile_names[t])
        N.append(north[t])
        E.append(east[t])

# Positions of the references
rf_n = [-39.74, -10.88]
rf_e = [36.26, 67.48]

# positions of 14 tiles used
n_14 = []
e_14 = []
for n in tiles_14:
    idx_n = np.where(np.array(tiles) == n)[0][0]
    e_14.append(E[idx_n])
    n_14.append(N[idx_n])

    
nice_fonts = {
        # Use LaTeX to write all text
        #"text.usetex": True,
        "font.family": "sans-serif",
        # Use 10pt font in plots, to match 10pt font in document
        "axes.labelsize": 10,
        "font.size": 10,
        # Make the legend/label fonts a little smaller
        "legend.fontsize": 6,
        "xtick.labelsize": 8,
        "ytick.labelsize": 8,
        "ytick.color" : "#696969",
        "xtick.color" : "#696969",
        "axes.labelcolor" : "#696969",
        "axes.edgecolor" : "#696969"
        }

plt.rcParams.update(nice_fonts)


fig, ax = plt.subplots(figsize=(3.6, 3.6))
#fig, ax = plt.subplots(figsize=(16, 16))
ax.scatter(E, N, marker='s', s=10.0, color='orange', label='MWA')
ax.scatter(e_14, n_14, marker='s', s=10.0, color='cornflowerblue', label='AUT')
ax.scatter(rf_e, rf_n, marker='s', s=10.0, color='crimson', label='REF')
#ax.scatter(-8.5, 0.9, marker='s', s=10.0, color='green', label='Center')

for i, txt in enumerate([r'$Ref_1$', r'$Ref_0$']):
    ax.annotate(txt, (rf_e[i], rf_n[i]),xytext=(1.28*rf_e[i],rf_n[i]), va='center', color='w', fontsize=7)

leg = ax.legend(loc="lower right", frameon=True, framealpha=0.5, markerscale=1, prop={'weight':'heavy'})
leg.get_frame().set_edgecolor('w')
leg.get_frame().set_facecolor('#222222')
for text in leg.get_texts():
    text.set_color('white')
for l in leg.legendHandles:
    l.set_alpha(1)

#ax.legend()
ax.set_aspect('equal')
ax.set_ylim(-100, 400)
ax.set_xlim(-200, 300)
ax.set_ylabel('North [m]')
ax.set_xlabel('East [m]')

plt.savefig(f'mwa_map.pdf', transparent=True, bbox_inches='tight')




# Force matplotlib to not use X-Server backend
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import matplotlib.pylab as pl
import seaborn as sns
import numpy as np

def plt_waterfall_pass(power, sat_id, start, stop, date, cmap, chs=None, good_ch=None, out_dir=None):
    '''Plot waterfall with sat window and occupied channels
    
    Args:
        power:          RF power array
        sat_id:         Norad cat ID
        start:          Start of epehm for sat_id
        stop:           Stop of ephem of sat_id
        chs:            Occupied channels [list]
        good_ch:        Most probable channel
        date:           Date of observation
    '''

    plt.style.use('dark_background')
    fig = plt.figure(figsize = (7,10))
    ax = fig.add_axes([0.12, 0.1, 0.72, 0.85])
    im = ax.imshow(power, interpolation='none', cmap=cmap)
    cax = fig.add_axes([0.88, 0.1, 0.03, 0.85])
    fig.colorbar(im, cax=cax)
    ax.set_aspect('auto')
    ax.set_title(f'Waterfall Plot: {sat_id} in {chs}')
    
    ax.set_xlabel('Freq Channel')
    ax.set_ylabel('Time Step')
    
    # horizontal highlight of ephem
    ax.axhspan(start, stop, alpha=0.1, color='white')
   
    if chs != None:
        # vertical highlight of channel
        for ch in chs:
            ax.axvspan(ch-1.0, ch+0.6, alpha=0.2, color='white')
    
    if good_ch != None:
        # highlight good channel
        ax.axvspan(good_ch-1.0, good_ch+0.6, alpha=0.6, edgecolor='#a7ff83', facecolor=None, fill=False)
    
    #plt.show()
    if chs != None:
        plt.savefig(f'{out_dir}/{date}_{sat_id}_{chs}_waterfall.png')
    else:
        plt.savefig(f'{out_dir}/{date}_{sat_id}_waterfall.png')

    plt.close()
    plt.rcParams.update(plt.rcParamsDefault)


def plt_channel_basic(
        out_dir, times, channel_power,
        chan_num, min_s, max_s, noise_threshold,
        arbitrary_threshold, sat_id, date):

    '''Plot power in channel, with various thresholds
    
    Args:
        times:          Time array
        channel_power:  Power in channel
        chan_num:       Channel Number
        min_s:          Minimum signal in channel_power
        max_s:          Maximum signal in channel_power
        noise_threshold: Noise Threshold (n*MAD)
        arbitrary_threshold: Arbitrary threshold used to only select bright passes
        sat_id:         Norad Cat ID
        date:           Date of observation
        '''
    
    
    plt.style.use('seaborn')
    
    # plt channel power
    plt.plot(times, channel_power, linestyle='-', linewidth=2, alpha=1.0, color='#db3751', label='Data')
    plt.fill_between(times, channel_power, color='#db3751', alpha=0.7)
    
    plt.axhline(arbitrary_threshold,alpha=1.0, linestyle='-', linewidth=2,
            color='#fba95f', label=f'Arbitrary Cut: {arbitrary_threshold} dBm')
    plt.axhspan(-1, arbitrary_threshold, color='#fba95f', alpha=0.4)
    
    plt.axhline(noise_threshold, linestyle='-', linewidth=2, color='#5cb7a9',
            label=f'Noise Cut: {noise_threshold:.2f} dBm')
    plt.axhspan(-1, noise_threshold, color='#5cb7a9', alpha=0.4)
    
    plt.ylim([min_s - 1, max_s + 1])
    plt.xlim([times[0], times[-1]])
    plt.ylabel('Power [dBm]')
    plt.xlabel('Time [s]')
    plt.title(f'Satellite Pass in Channel: [{chan_num}]')
    plt.tight_layout()
    leg = plt.legend(frameon=True)
    leg.get_frame().set_facecolor('grey')
    leg.get_frame().set_alpha(0.2)
    for l in leg.legendHandles:
        l.set_alpha(1)
    plt.savefig(f'{out_dir}/{date}_{sat_id}_{chan_num}_channel.png')
    plt.close()
    plt.rcParams.update(plt.rcParamsDefault)




def sat_plot(out_dir, ids, norad_id, alt, az, num_passes, date, name):
    '''Plots satellite passes
    
    Args:
        alt: list of altitude values
        az: list of azimuth values
        num_passes: Number of satellite passes
    '''
    
    
    # Set up the polar plot.
    plt.style.use('seaborn')
    plt.style.use('dark_background')
    figure = plt.figure(figsize=(7,6))
    ax = figure.add_subplot(111, polar=True)
    ax.set_ylim(90, 0)
    ax.set_rgrids([0,30,60,90], angle=22)
    ax.set_theta_zero_location('N')
    ax.set_theta_direction(-1)
    ax.set_title(f'{num_passes} satellite passes in {date} [{norad_id}] window', y=1.05)
    ax.grid(color='grey', linewidth=1.6, alpha=0.5)

    colors = pl.cm.Spectral(np.linspace(0.17,0.9,len(alt)))
    
    for i in range(len(alt)):
        plt.plot(az[i], alt[i], '-', linewidth=2.4, alpha=0.8, color=colors[i], label=f'{ids[i]}')
        plt.legend()
    
    leg = plt.legend(frameon=True, bbox_to_anchor=(0.28, 1.0, 1., -0.95), loc='center right', title="Norad SatID")
    leg.get_frame().set_facecolor('grey')
    leg.get_frame().set_alpha(0.4)
    for l in leg.legendHandles:
        l.set_alpha(1)
    
    plt.tight_layout()
    plt.savefig(f'{out_dir}/{date}_{norad_id}_{name}.png')
    plt.close()
    plt.rcParams.update(plt.rcParamsDefault)






import os
import sys
import time
import argparse
import numpy as np
sys.path.append('../decode_rf_data')
import matplotlib.pyplot as plt
from colormap import spectral
import rf_data as rf

parser = argparse.ArgumentParser(description="""
        Will plot waterfall plots
        """)

parser.add_argument('--rf_dir', metavar='\b', default='./../../data/', help='Path to rf data directory. Default=./../../data/')
parser.add_argument('--ref_name', metavar='\b', default='rf0XX_2019-10-10-02:30', help='Name of ref data file. Default=rf0XX_2019-10-10-02:30')
parser.add_argument('--tile_name', metavar='\b', default='S10XX_2019-10-10-02:30', help='Name of tile data file. Default=S10XX_2019-10-10-02:30')
parser.add_argument('--out_dir', metavar='\b', default='./../../outputs/paper_plots/', help='Output dir. Default=./../../outputs/paper_plots/')

args = parser.parse_args()
rf_dir = args.rf_dir
ref_name = args.ref_name
tile_name = args.tile_name
out_dir = args.out_dir

# Make output dir if it doesn't exist
os.makedirs(os.path.dirname(out_dir), exist_ok=True)




# read in raw data
ref_p, ref_t = rf.read_data(f'{rf_dir}/{ref_name}.txt')
tile_p, tile_t = rf.read_data(f'{rf_dir}/{tile_name}.txt')

# scale median to zero
ref_p_median = np.median(ref_p)
tile_p_median = np.median(tile_p)
r_image = ref_p - ref_p_median
t_image = tile_p - tile_p_median

# setting dynamic range of waterfall to be 30 dB above the median
vmin = 0
vmax = 30

# Custom spectral colormap
cmap = spectral()

nice_fonts = {
        # Use LaTeX to write all text
        "text.usetex": True,
        "font.family": "sans-serif",
        # Use 10pt font in plots, to match 10pt font in document
        "axes.labelsize": 10,
        "font.size": 10,
        # Make the legend/label fonts a little smaller
        "legend.fontsize": 8,
        "xtick.labelsize": 8,
        "ytick.labelsize": 8,
        #"ytick.color" : "w",
        #"xtick.color" : "w",
        #"axes.labelcolor" : "w",
        #"axes.edgecolor" : "w"
        }

plt.rcParams.update(nice_fonts)

fig = plt.figure(figsize=(7, 4.2))
#fig, axs = plt.subplots(1,2, figsize=(7,5))
#axs = axs.ravel()

ax1 = fig.add_axes([0.00, 0.0, 0.46, 1])
ax2 = fig.add_axes([0.50, 0.0, 0.46, 1])
cax = fig.add_axes([0.98, 0.0, 0.015,1])

ax1.imshow(t_image, vmin=vmin, vmax=vmax, interpolation='none', cmap=cmap)
ax1.set_aspect('auto')
image = ax1.get_images()[0]
cbar = fig.colorbar(image, cax=cax, label='Power [dB]')

# Number of time steps on y-axis
number_t = 5
t_step = int(len(tile_t)/(number_t-1))
times = list(tile_t)
times = times[::t_step]

t_tz = []

# Convert UNIX time to local HH:MM time
for i in range(len(times)):
    
    perth_t = float(times[i])+28800 #28800=+8GMT @ PERTH
    hms = time.strftime('%H:%M', time.gmtime(perth_t))
    t_tz.append(hms)

# Frequency: x-axis
start_freq = 137.15
stop_freq = 138.55

# X-axis stuff
x_ax = t_image.shape[1]
freqs = np.arange(start_freq, stop_freq, 0.25)
x_ticks = np.arange(0, x_ax, (0.25/0.0125)) #.0125MHz/ch
ax1.set_xticks(x_ticks)
ax1.set_xticklabels(freqs)
ax1.set_xlabel('Freqency [MHz]')

# Y-axis stuff
y_ax = t_image.shape[0]
y_ticks = np.arange(0, y_ax, t_step)
ax1.set_yticks(y_ticks)
ax1.set_yticklabels(t_tz)
ax1.set_ylabel('MWA local time [HH:MM]')


ax2.imshow(r_image, vmin=vmin, vmax=vmax, interpolation='none', cmap=cmap)
ax2.set_aspect('auto')
# Number of time steps on y-axis
number_t = 5
t_step = int(len(ref_t)/(number_t-1))
times = list(ref_t)
times = times[::t_step]

t_tz = []

# Convert UNIX time to local HH:MM time
for i in range(len(times)):
    
    perth_t = float(times[i])+28800 #28800=+8GMT @ PERTH
    hms = time.strftime('%H:%M', time.gmtime(perth_t))
    t_tz.append(hms)

# Frequency: x-axis
start_freq = 137.15
stop_freq = 138.55

# X-axis stuff
x_ax = r_image.shape[1]
freqs = np.arange(start_freq, stop_freq, 0.25)
x_ticks = np.arange(0, x_ax, (0.25/0.0125)) #.0125MHz/ch
ax2.set_xticks(x_ticks)
ax2.set_xticklabels(freqs)
ax2.set_xlabel('Freqency [MHz]')

# Y-axis stuff
ax2.set_yticklabels([])
ax2.set_yticks([])


plt.savefig(f'{out_dir}/waterfall.png', dpi=600, bbox_inches='tight')
#plt.savefig(f'waterfall.png', dpi=600, transparent=True, bbox_inches='tight')




import numpy as np
from align_data import savgol_interp
import concurrent.futures

from pathlib import Path
from itertools import repeat
from itertools import product
from datetime import datetime, timedelta
    
import sys
sys.path.append('../decode_rf_data')
import rf_data as rf


def save_aligned(
        ref, aut,
        ref_path,
        aut_path,
        savgol_window_1, 
        savgol_window_2, 
        polyorder, 
        interp_type, 
        interp_freq,
        out_dir,
        date,
        time_stamp):

    try:
        ref_file = f'{ref_path}/{ref}_{time_stamp}.txt'
        aut_file = f'{aut_path}/{aut}_{time_stamp}.txt'

        _, _, _, _, ref_p_aligned, tile_p_aligned, time_array = savgol_interp(
                ref_file,
                aut_file,
                savgol_window_1=savgol_window_1,
                savgol_window_2=savgol_window_2,
                polyorder=polyorder,
                interp_type=interp_type,
                interp_freq=interp_freq
                )

        # creates output directory if it doesn't exist
        save_dir = Path(f'{out_dir}/{date}/{time_stamp}')
        save_dir.mkdir(parents=True, exist_ok=True)
        
        # Convert the power array to float32
        # Convert list of times to float64 (double)
        # Save as compressed npz file. Seems to drastically reduce size
        np.savez_compressed(f'{save_dir}/{ref}_{aut}_{time_stamp}_aligned.npz',
                ref_p_aligned=np.single(ref_p_aligned),
                tile_p_aligned=np.single(tile_p_aligned),
                time_array=np.double(time_array)
                )

        return f'Saving {ref}_{aut}_{time_stamp}_aligned.npz'

    except Exception:
        return f'Cound not save {ref}_{aut}_{time_stamp}_aligned.npz. Missing file'
        # This exception should only be raised if both files don't exist
        # In that case, the exception from rf_data.read_data() will be 
        # displayed here.


if __name__ == '__main__':

    import argparse

    parser = argparse.ArgumentParser(description="""
        Time alignes and smoothes all data 
        between two date ranges. Saves data
        pairs in organised directory 
        structure as .npz files.
        """)
    
    parser.add_argument('--data_dir', metavar='\b', help='Dir where date is saved')
    parser.add_argument('--start_date', metavar='\b', help='Date from which to start aligning data. Ex: 2019-10-10')
    parser.add_argument('--stop_date', metavar='\b', help='Date until which to align data. Ex: 2019-10-11')
    parser.add_argument('--out_dir', metavar='\b', default='./../../outputs/align_data/',help='Output directory. Default=./../../outputs/align_data/')
    parser.add_argument('--savgol_window_1', metavar='\b', type=int,  default=11,help='Length of savgol window. Must be odd. Default=11')
    parser.add_argument('--savgol_window_2', metavar='\b', type=int,  default=15,help='Length of savgol window. Must be odd. Default=15')
    parser.add_argument('--polyorder', metavar='\b', type=int,  default=2,help='Order of polynomial to fit to savgol window. Default=2')
    parser.add_argument('--interp_type', metavar='\b', default='cubic',help='Type of interpolation. Ex: cubic, linear, etc. Default=cubic')
    parser.add_argument('--interp_freq', metavar='\b', type=int,  default=1,help='Frequency at which to resample smoothed data, in Hertz. Default=1')
    
    
    args = parser.parse_args()
    
    data_dir =            args.data_dir
    start_date =          args.start_date
    stop_date =           args.stop_date
    out_dir =             args.out_dir
    savgol_window_1 =     args.savgol_window_1
    savgol_window_2 =     args.savgol_window_2
    polyorder =           args.polyorder
    interp_type =         args.interp_type
    interp_freq =         args.interp_freq
    
    
    # Save logs 
    Path(out_dir).mkdir(parents=True, exist_ok=True)
    sys.stdout = open(f'{out_dir}/logs_{start_date}_{stop_date}.txt', 'a')
    
    # Import list of tile names from rf_data.py
    tiles = rf.tile_names()
    
    refs = tiles[:4]
    AUTS = tiles[4:]
    
    # Split the list into XX and YY lists
    refs_XX = refs[::2]
    refs_YY = refs[1::2]
    
    AUTS_XX = AUTS[::2]
    AUTS_YY = AUTS[1::2]
    
    
    # Create a list of pairs of tiles to be aligned. 
    # Essentially all possible combinations of Refs, AUTS
    align_pairs = []
    for pair in list(product(refs_XX, AUTS_XX)):
        align_pairs.append(pair)
    
    for pair in list(product(refs_YY, AUTS_YY)):
        align_pairs.append(pair)
    
    
    # Time stuff to help traverse data dir tree.
    t_start = datetime.strptime(start_date, '%Y-%m-%d')
    t_stop = datetime.strptime(stop_date, '%Y-%m-%d')
    n_days = (t_stop - t_start).days
    
    dates = []
    date_time = []
    
    for i in range(n_days+1):
        day = t_start + timedelta(days=i)
        date = day.strftime('%Y-%m-%d')
        dates.append(date)
        d_t = []
        
        for j in range(48):
            t_delta = datetime.strptime(date,'%Y-%m-%d') + timedelta(minutes=30*j)
            d_time = t_delta.strftime('%Y-%m-%d-%H:%M')
            d_t.append(d_time)
    
        date_time.append(d_t)

    
    data_dir = Path(data_dir)
    out_dir = Path(out_dir)
    
    
    
    for pair in align_pairs:
        for d in range(len(dates)):
            ref = pair[0]
            aut = pair[1]
        
            ref_path = data_dir/ref/dates[d]
            aut_path = data_dir/aut/dates[d]
           
            # Parallization magic happens here
            with concurrent.futures.ProcessPoolExecutor() as executor:
                results = executor.map(
                        save_aligned, 
                        repeat(ref), 
                        repeat(aut),
                        repeat(ref_path),
                        repeat(aut_path),
                        repeat(savgol_window_1), 
                        repeat(savgol_window_2), 
                        repeat(polyorder), 
                        repeat(interp_type), 
                        repeat(interp_freq), 
                        repeat(out_dir),
                        repeat(dates[d]),
                        date_time[d])
    
    
            for result in results:
                print(result)
    
    



import os
import json
import sat_ids
import concurrent.futures
from itertools import repeat
from sat_ephemeris import sat_plot


def plot_ephem(json_dir, out_dir, sat_id):
    try:
        with open(f'{json_dir}/{sat_id}.json', 'r') as ephem:
            sat_ephem = json.load(ephem)
            altitude = sat_ephem['sat_alt']
            azimuth = sat_ephem['sat_az']
            num_passes = len(sat_ephem['time_array'])
            
            plt = sat_plot(sat_id, altitude, azimuth, num_passes)
            plt.savefig(f'{out_dir}/{sat_id}.png')
            plt.close()

    except Exception:
        print(f'Could not plot {sat_id}.png. Source json file missing ')

if __name__ == '__main__':
    import argparse
    
    parser = argparse.ArgumentParser(description=""" 
            Plot Satellite passes from json ephemeris files.
            """)
    
    parser.add_argument('--out_dir', metavar='\b', default='./../../outputs/sat_ephemeris/ephem_plots/', help='Path to output directory. Default=./../../outputs/sat_ephemeris/ephem_plots/')
    parser.add_argument('--json_dir', metavar='\b', default='./../../outputs/sat_ephemeris/ephem_json/', help='Path to ephem_json directory. Default=./../../outputs/sat_ephemeris/ephem_json/')
    
    args = parser.parse_args()
    
    out_dir =   args.out_dir
    json_dir =  args.json_dir
    
    # Create output directory
    os.makedirs(os.path.dirname(out_dir), exist_ok=True) 
    
    # Load list of satellites (Nodar ids)
    sat_list = list(sat_ids.norad_ids.values())
    
    # Parellization Magic Here!
    with concurrent.futures.ProcessPoolExecutor() as executor:
        results = executor.map(
                plot_ephem, 
                repeat(json_dir),
                repeat(out_dir),
                sat_list)
    



import sys
import numpy as np
import healpy as hp
import scipy.optimize as opt
import numpy.polynomial.polynomial as poly
from mwa_pb.mwa_sweet_spots import all_grid_points

sys.path.append('../decode_rf_data')
from colormap import spectral, jade, kelp

# Custom spectral colormap
jade, _ = jade()

def hp_slices_horizon(nside=None, zenith_angle=90):
    '''Healpix pix indices of NS, EW slices and above horizon'''
    # theta phi values of each pixel 
    hp_indices = np.arange(hp.nside2npix(nside))
    θ, ɸ = hp.pix2ang(nside, hp_indices)

    # healpix indices above the horizon
    above_horizon_indices = np.where(θ <= np.radians(zenith_angle))[0]
    
    # pixel coords above the horizon
    ɸ_above_horizon = ɸ[above_horizon_indices]

    NS_indices = []
    EW_indices = []

    # pixel indices along N, E, S, W slices
    # order the indices such that they proceed from N -> S or E -> W
    n_slice = sorted(np.where((np.round(np.degrees(ɸ_above_horizon))) ==  45)[0])
    e_slice = sorted(np.where((np.round(np.degrees(ɸ_above_horizon))) == 135)[0])
    s_slice = sorted(np.where((np.round(np.degrees(ɸ_above_horizon))) == 225)[0], reverse=True)
    w_slice = sorted(np.where((np.round(np.degrees(ɸ_above_horizon))) == 315)[0], reverse=True)

    NS_indices.extend(s_slice)
    NS_indices.extend(n_slice)
    EW_indices.extend(w_slice)
    EW_indices.extend(e_slice)

    return [NS_indices, EW_indices, above_horizon_indices]


def slice_map(hp_map, za):
    '''slices healpix map along NS, EW'''
    
    NS_indices, EW_indices, _ = hp_slices_horizon(nside, zenith_angle=za)

    θ_NS, ɸ_NS = np.degrees(hp.pix2ang(nside, NS_indices))
    θ_EW, ɸ_EW = np.degrees(hp.pix2ang(nside, EW_indices))

    zenith_angle_NS = []
    for i, j in zip(θ_NS, ɸ_NS):
        if j <= 180:
            zenith_angle_NS.append(-1*i)
        else:
            zenith_angle_NS.append(i)
    
    zenith_angle_EW = []
    for i, j in zip(θ_EW, ɸ_EW):
        if j <= 180:
            zenith_angle_EW.append(-1*i)
        else:
            zenith_angle_EW.append(i)
    
    NS_data = [hp_map[NS_indices], zenith_angle_NS]
    EW_data = [hp_map[EW_indices], zenith_angle_EW]

    return [NS_data, EW_data]

def nan_mad(ref_map):
    '''Compute mad while ignoring nans'''
    ref_map_mad = []
    for j in ref_map:
        if j != []:
            j = np.asarray(j)
            j = j[~np.isnan(j)]
            ref_map_mad.append(mad(j))
        else:
            ref_map_mad.append(np.nan)

    ref_map_mad = np.asarray(ref_map_mad)
    ref_map_mad[np.where(ref_map_mad == np.nan)] = np.nanmean(ref_map_mad)

    return ref_map_mad


def tile_map_slice(good_map, za):
    '''slices ref healpix map along NS & EW'''
    
    ref_map_NS, ref_map_EW = slice_map(np.asarray(good_map), za)

    ref_med_map_NS = np.asarray([(np.nanmedian(i) if i != [] else np.nan) for i in ref_map_NS[0]])
    ref_mad_map_NS = np.asarray(nan_mad(ref_map_NS[0]))
    za_NS = ref_map_NS[1]

    ref_med_map_EW = np.asarray([(np.nanmedian(i) if i != [] else np.nan) for i in ref_map_EW[0]])
    ref_mad_map_EW = np.asarray(nan_mad(ref_map_EW[0]))
    za_EW = ref_map_EW[1]
    
    NS_data = [ref_med_map_NS, ref_mad_map_NS, za_NS]
    EW_data = [ref_med_map_EW, ref_mad_map_EW, za_EW]

    return [NS_data, EW_data]


# rotate func written by Jack Line
def rotate(nside, angle=None,healpix_array=None,savetag=None,flip=False):
    '''Takes in a healpix array, rotates it by the desired angle, and saves it.
    Optionally flip the data, changes east-west into west-east because
    astronomy'''

    # theta phi values of each pixel 
    hp_indices = np.arange(hp.nside2npix(nside))
    θ, ɸ = hp.pix2ang(nside, hp_indices)

    new_hp_inds = hp.ang2pix(nside,θ,ɸ+angle)

    ##Flip the data to match astro conventions
    if flip == True:
        new_angles = []
        for phi in ɸ:
            if phi <= np.pi:
                new_angles.append(np.pi - phi)
            else:
                new_angles.append(3*np.pi - phi)
        new_hp_inds = hp.ang2pix(nside,ɸ, np.asarray(new_angles))

    ##Save the array in the new order
    if savetag:
        np.savez_compressed(savetag,beammap=healpix_array[new_hp_inds])

    return healpix_array[new_hp_inds]

# chisquared minimization to best fit map to data
def fit_gain(map_data=None, map_error=None,beam=None):
    '''Fit the beam model to the measured data using
    chisquared minimization'''
    
    bad_values = np.isnan(map_data)
    map_data = map_data[~bad_values]
    map_error = map_error[~bad_values]

    map_error[np.where(map_error == 0)] = np.mean(map_error)

    def chisqfunc(gain):
        model = beam[~bad_values] + gain
        chisq = sum((map_data - model)**2)
        #chisq = sum(((map_data - model)/map_error)**2)
        return chisq

    x0 = np.array([0])

    result =  opt.minimize(chisqfunc,x0)
    
    return result.x


def poly_fit(x, y, map_data, order):
    '''Fit polynominal of order to data'''
    
    x = np.asarray(x)
    y = np.asarray(y)

    bad_values = np.isnan(map_data)
    x_good = x[~bad_values]
    y_good = y[~bad_values]
    coefs = poly.polyfit(x_good, y_good, order)
    fit = poly.polyval(x, coefs)
    return fit


def plt_slice(
        fig=None, sub=None,
        zen_angle=None, map_slice=None,
        map_error=None, model_slice=None, 
        delta_pow=None, pow_fit=None, 
        slice_label=None, model_label=None):

    '''Plot a slice of the beam, with measured
    data, errorbars, and fit the simulated beam
    to the data. Also plot the diff b/w data and
    the model'''

    ax = fig.add_subplot(sub)

    ax.errorbar(
           zen_angle, map_slice, yerr=map_error, 
           fmt='.', color='#326765', ecolor='#7da87b',
           elinewidth=1.2, capsize=1.2, capthick=1.4,
           alpha=0.9, label=slice_label)

    ax.plot(zen_angle,model_slice, color='#c70039', linewidth=1.2, alpha=0.9, label=model_label)

    ax.set_ylabel('Power (dB)')
    #ax.set_ylim(bottom=-30)
    ax.set_xlabel('Zenith Angle (degrees)')
    ax.legend()
    ax.set_xlim([-92,92])
    ax.set_ylim([-50,5])

    divider = make_axes_locatable(ax)
    dax = divider.append_axes("bottom", size="30%", pad=0.1)

    #dax = fig.add_subplot(2,1,2)
    dax.scatter(zen_angle, delta_pow, marker='.', color='#27296d')
    dax.plot(zen_angle, pow_fit, linewidth=1.2, alpha=0.9, color='#ff8264')
    dax.set_ylabel('Data - Model (dB)')
    dax.set_ylim([-10,10])
    #dax.set_xticklabels([])
    dax.set_xlim([-92,92])

    return ax

def plot_healpix(data_map=None, fig=None, sub=None,title=None,vmin=None,vmax=None,cmap=None, cbar=True):
    '''Yeesh do some healpix magic to plot the thing'''
    
    # Disable cryptic healpy warnings. Can't figure out where they originate
    import warnings
    warnings.filterwarnings("ignore", category=RuntimeWarning) 
    
    hp.orthview(
            map=data_map,coord='E', fig=fig,
            half_sky=True, rot=(0,90,180), xsize=600,
            title=title,sub=sub,min=vmin,max=vmax,
            cmap=cmap,notext=True, hold=True, cbar=cbar)

    hp.graticule(dpar=10,coord='E',color='k',alpha=0.3,dmer=45)
   
    # Altitude grid
    hp.projtext(00.0*(np.pi/180.0), 225.0*(np.pi/180),'0', color='w', coord='E')
    hp.projtext(30.0*(np.pi/180.0), 225.0*(np.pi/180),'30', color='w', coord='E')
    hp.projtext(60.0*(np.pi/180.0), 225.0*(np.pi/180),'60', color='w', coord='E')

    # NSEW 
    hp.projtext(80.0*(np.pi/180.0), 000.0*(np.pi/180.0), r'$N  $', coord='E',color='w', verticalalignment='top')
    hp.projtext(80.0*(np.pi/180.0), 090.0*(np.pi/180.0), r'$E  $', coord='E',color='w', horizontalalignment='right')
    hp.projtext(80.0*(np.pi/180.0), 180.0*(np.pi/180.0), r'$S  $', coord='E',color='w', verticalalignment='bottom')
    hp.projtext(80.0*(np.pi/180.0), 270.0*(np.pi/180.0), r'$W  $', coord='E',color='w', horizontalalignment='left')
    

def beam_slice(f):

    f_name, _ = f.name.split('.')
    t_name, r_name, _, _ = f_name.split('_')

    pointings = ['0','2','4','41']


    # load data from map .npz file
    tile_map = np.load(f, allow_pickle=True)
    fee_m = np.load(fee_map, allow_pickle=True)
    
    for p in pointings:

        tile = tile_map[p]

        if 'XX' in t_name:
            fee = fee_m[p][0]
        else:
            fee = fee_m[p][1]
        
        # rotate maps so slices can be taken 
        fee_r = rotate(nside, angle= -np.pi/4, healpix_array=fee)
        tile_r = rotate(nside, angle=-np.pi/4, healpix_array=tile)

        # slice the tile and fee maps along NS, EW
        # zenith angle thresh of 70 to determine fit gain factor
        NS_f, EW_f = slice_map(fee_r, 70)
        NS_t, EW_t = tile_map_slice(tile_r, 70)

        gain_NS = fit_gain(map_data=NS_t[0], map_error=NS_t[1], beam=NS_f[0])
        gain_EW = fit_gain(map_data=EW_t[0], map_error=EW_t[1], beam=EW_f[0])

        # slice the tile and fee maps along NS, EW. 
        # the above gain factor is applied to full beam slices
        NS_fee, EW_fee = slice_map(fee_r, 90)
        NS_tile, EW_tile = tile_map_slice(tile_r, 90)

        # Scale the data so that it best fits the beam slice
        NS_tile_med = NS_tile[0] - gain_NS[0]
        EW_tile_med = EW_tile[0] - gain_EW[0]
        
        # delta powers
        del_NS = NS_tile_med - NS_fee[0]
        del_EW = EW_tile_med - EW_fee[0]

        # 3rd order poly fits for residuals
        fit_NS = poly_fit(NS_tile[2], del_NS, NS_tile[0], 3)  
        fit_EW = poly_fit(EW_tile[2], del_EW, EW_tile[0], 3)  

        # Visualize the tile map and diff map
        # healpix meadian map
        tile_med = np.asarray([(np.nanmedian(j) if j != [] else np.nan ) for j in tile])
        
        residuals = tile_med - fee
        residuals[np.where(fee < -30)] = np.nan
        residuals[np.where(tile_med == np.nan)] = np.nan


        # This is an Awesome plot
        plt.style.use('seaborn')
        fig1 = plt.figure(figsize=(10, 8))

        ax1 = plt.subplot(2,2,1)
        plt_slice(
                fig=fig1, sub=221,
                zen_angle=NS_tile[2], map_slice=NS_tile_med,
                map_error=NS_tile[1], model_slice=NS_fee[0],
                delta_pow=del_NS, pow_fit=fit_NS, 
                slice_label='Tile NS', model_label='FEE NS')

        ax2 = fig1.add_axes([0.48, 0.52, 0.48, 0.43])
        plot_healpix(data_map=tile_med, sub=(2,2,2), fig=fig1, title='tile map', cmap=jade, vmin=-50, vmax=0, cbar=False)
        ax7 = plt.gca()
        image = ax7.get_images()[0]
        cax = fig1.add_axes([0.92, 0.52, 0.015, 0.43])
        cbar = fig1.colorbar(image, cax=cax, label='dB')
        
        ax3 = plt.subplot(2,2,3)
        plt_slice(
                fig=fig1, sub=223,
                zen_angle=EW_tile[2], map_slice=EW_tile_med,
                map_error=EW_tile[1], model_slice=EW_fee[0],
                delta_pow=del_EW, pow_fit=fit_EW, 
                slice_label='Tile EW', model_label='FEE EW')
        
        ax4 = fig1.add_axes([0.48, 0.02, 0.48, 0.43])
        plot_healpix(data_map=residuals, sub=(2,2,4), fig=fig1, title='diff map', cmap='inferno', vmin=-10, vmax=5,  cbar=False)
        ax8 = plt.gca()
        image = ax8.get_images()[0]
        cax = fig1.add_axes([0.92, 0.02, 0.015, 0.43])
        cbar = fig1.colorbar(image, cax=cax, label='dB')

        plt.tight_layout()
        plt.savefig(f'{out_dir}/{p}/{t_name}_{r_name}_{p}_beam_slices.png')
        plt.close()


if __name__=='__main__':
    
    import argparse
    import numpy as np
    from pathlib import Path
    import concurrent.futures
    import matplotlib
    matplotlib.use('Agg')
    import matplotlib.pyplot as plt
    import matplotlib.gridspec as gs
    from mpl_toolkits.axes_grid1 import make_axes_locatable
    from scipy.stats import median_absolute_deviation as mad

    import sys
    sys.path.append('../decode_rf_data')
    from colormap import spectral
    
    # Custom spectral colormap
    cmap = spectral()
    
    parser = argparse.ArgumentParser(description="""
        Plot healpix map of reference data
        """)
    
    parser.add_argument('--out_dir', metavar='\b', default='../../outputs/tile_maps/compare_beams/',
            help='Output directory. Default=../../outputs/tile_maps/compare_beams/')
    parser.add_argument('--map_dir', metavar='\b', default='../../outputs/tile_maps/tile_maps_norm/',
            help='Tile map directory. Default=../../outputs/tile_maps/tile_maps_norm/')
    parser.add_argument('--fee_map', metavar='\b', default='../../outputs/tile_maps/FEE_maps/mwa_fee_beam.npz',
            help='Healpix FEE map of mwa tile. default=../../outputs/tile_maps/FEE_maps/mwa_fee_beam.npz')
    parser.add_argument('--nside', metavar='\b',type=int, default=32,help='Healpix Nside. Default = 32')
    
    
    args = parser.parse_args()
    
    out_dir     = Path(args.out_dir)
    map_dir     = Path(args.map_dir)
    fee_map     = args.fee_map
    nside       = args.nside
    
    # make output dir if it doesn't exist
    out_dir.mkdir(parents=True, exist_ok=True)

    # MWA beam pointings
    pointings = ['0', '2', '4', '41']

    # Create output directory structure
    for p in pointings:
        Path(f'{out_dir}/{p}/').mkdir(parents=True, exist_ok=True)

    # find all map files
    map_files = [item for item in map_dir.glob('*.npz')]

    # Parallization magic happens here
    with concurrent.futures.ProcessPoolExecutor() as executor:
        results = executor.map(beam_slice, map_files)
#    beam_slice(map_files[0])



import sys
import numpy as np
import matplotlib.pyplot as plt
sys.path.append('../align_data')
from align_data import savgol_interp

ch =8
ref_t, ref_p, tile_t, tile_p, ref_p_aligned, tile_p_aligned, time_array = savgol_interp(
        '../../../tiles_data/rf0XX/2019-09-15/rf0XX_2019-09-15-11:00.txt',
        '../../../tiles_data/S06XX/2019-09-15/S06XX_2019-09-15-11:00.txt',
        savgol_window_1 =11,
        savgol_window_2 =15,
        polyorder=2,
        interp_type='cubic',
        interp_freq=1
        )

plt.style.use('seaborn')

nice_fonts = {
        # Use LaTeX to write all text
        #"text.usetex": True,
        "font.family": "sans-serif",
        # Use 10pt font in plots, to match 10pt font in document
        "axes.labelsize": 10,
        "font.size": 10,
        # Make the legend/label fonts a little smaller
        "legend.fontsize": 6,
        "xtick.labelsize": 8,
        "ytick.labelsize": 8,
        }

plt.rcParams.update(nice_fonts)


fig = plt.figure(figsize=(3.6,2.4))

tile_t = tile_t - tile_t[0]
time_array = time_array - time_array[0]

med = np.median(tile_p)
tile_p =tile_p - med
tile_p_aligned = tile_p_aligned - med

plt.plot(time_array,tile_p_aligned[::, ch], linewidth=1, color='#2c5d63', alpha=0.9, label='SavGol')
plt.scatter(tile_t, tile_p[::, ch], color='#7fa998',marker='.', s=2, alpha=0.6, label='AUT raw')

leg = plt.legend(loc="upper right", frameon=True, markerscale=4, handlelength=1)
leg.get_frame().set_facecolor('white')
for l in leg.legendHandles:
    l.set_alpha(1)

plt.ylabel('Power [dB]')
plt.xlabel('Time [s]')
plt.tight_layout()
#plt.savefig(f'savgol.png', dpi=600, bbox_inches='tight')
plt.savefig(f'./../../outputs/paper_plots/savgol.pdf', bbox_inches='tight')



import sys
import math
import json
import pytz
import json
import numpy as np
from pathlib import Path
from astropy.time import Time
from datetime import datetime, timedelta



def time_tree(start_date, stop_date, time_zone):
    "Start/end of 30 minuts obs in local, unix, gps"

    # The time input is in local time. As in Austraila/Perth
    local = pytz.timezone(time_zone)
    
    t_start = datetime.strptime(start_date, '%Y-%m-%d')
    t_stop = datetime.strptime(stop_date, '%Y-%m-%d')
    
    # Number of days that the date range spans
    n_days = (t_stop - t_start).days
    
    # YYYY-MM-DD-HH:MM format
    obs_time = []
    
    # Start of half hour obs in unix time
    obs_unix = []
    
    # End of half hour obs in unix time
    obs_unix_end = []
    
    # The +1 makes the date ranges inclusive
    for i in range(n_days+1):
        day = t_start + timedelta(days=i)
        date = day.strftime('%Y-%m-%d')
        
        # loop over 48x30 minute obs in a day
        for j in range(48):
            t_delta = datetime.strptime(date,'%Y-%m-%d') + timedelta(minutes=30*j)
            # convert t_delta to a readable string YYYY-MM-DD-HH:MM
            d_time = t_delta.strftime('%Y-%m-%d-%H:%M')
            
            # Convert from naive local time to utc aware time
            utc_delta = local.localize(t_delta, is_dst=None).astimezone(pytz.utc)
            
            # convert to a unix timestamp, used within rf explorer data files
            utc_unix = utc_delta.timestamp()
            # time at end of half hour window
            utc_unix_end = utc_unix + (30 * 60)
            
            obs_time.append(d_time)
            obs_unix.append(utc_unix)
            obs_unix_end.append(utc_unix_end)
    
    # Start and end of 30 min obs in gps time
    # Round to nearest int
    obs_gps     = np.rint(Time(obs_unix, format='unix').gps)
    obs_gps_end = np.rint(Time(obs_unix_end, format='unix').gps)

    return (obs_time, obs_unix, obs_unix_end, obs_gps, obs_gps_end)


def read_json(out_dir, f_name):
    """Read ultimate_pointing_times.json"""

    with open(f'{out_dir}/{f_name}') as table:
        data = json.load(table)
        pointings   = data['grid_pt'] 
        start_gps   = data['start_gps']
        stop_gps    = data['stop_gps'] 
        obs_length  = data['obs_length']

    return (pointings, start_gps, stop_gps, obs_length)


def time_filter(s_rise, s_set, times):
    """How much of the 30 minute
    window was at a particular 
    pointing"""
    
    # I. sat rises before times, sets within times window
    if (s_rise < times[0] and s_set > times[0] and s_set <= times[-1]):
        occu = (s_set - times[0])/1800

    
    # II. sat rises and sets within times
    elif (s_rise >= times[0] and s_set <= times[-1]):
        occu = (s_set - s_rise)/1800
    
    # III. sat rises within times, and sets after
    elif (s_rise >= times[0] and s_rise < times[-1] and s_set > times[-1]):
        occu = (times[-1] - s_rise)/1800
    
    # IV. sat rises before times and sets after
    elif (s_rise < times[0] and s_set > times[-1]):
        occu = (times[-1] - times[0])/1800
   
    # V. sat completely out of times. Could be on either side
    else:
        occu = None
    
    return occu


def obs_pointing(
        obs_time, 
        obs_gps, 
        obs_gps_end, 
        pointings,
        start_gps,
        stop_gps,
        start_date, 
        stop_date,
        out_dir
        ):

    """Find pointing of 30 min obs
    Classification of more than 70%
    of an obs is at one pointing"""
    
    point_0 = []
    point_2 = []
    point_4 = []
    point_41 = []
    
    # loop over each 30 min observation
    for i in range(len(obs_time)):
    
        # start, end times in gps format
        obs_window = [obs_gps[i], obs_gps_end[i]]
    
        # loop over mwa pointing observations from 
        # ultimate_pointing_times.json
        for j in range(len(start_gps)):
            occu = time_filter(start_gps[j], stop_gps[j], obs_window)
    
            if (occu != None and occu >= 0.7):
                if pointings[j] == 0:
                    point_0.append(obs_time[i])
                elif pointings[j] == 2:
                    point_2.append(obs_time[i])
                elif pointings[j] == 4:
                    point_4.append(obs_time[i])
                elif pointings[j] == 41:
                    point_41.append(obs_time[i])
                else:
                    pass


    # Create dictionary to be saved to json
    obs_pointings = {}
    
    obs_pointings['start_date'] = start_date
    obs_pointings['stop_date']  = stop_date
    obs_pointings['point_0']    = point_0
    obs_pointings['point_2']    = point_2
    obs_pointings['point_4']    = point_4
    obs_pointings['point_41']   = point_41
    
    
    with open(f'{out_dir}/obs_pointings.json', 'w') as outfile:
        json.dump(obs_pointings, outfile, indent=4)



if __name__ == "__main__":
    
    import argparse
    
    parser = argparse.ArgumentParser(description="""
            Classify the pointings of 30 min observations
            """)
    
    parser.add_argument('--start_date', metavar='\b', help='Date from which to determine sat ephemeris. Ex: 2019-10-10')
    parser.add_argument('--stop_date', metavar='\b', help='Date until which to determine sat ephemeris. Ex: 2019-10-11')
    parser.add_argument('--time_zone', metavar='\b', default='Australia/Perth', help='Time zone where data was recorded. Default=Australia/Perth')
    parser.add_argument('--out_dir', metavar='\b', default='./../../outputs/beam_pointings/', help='Directory where json metadata files live. Default=./../../outputs/beam_pointings/')
    parser.add_argument('--f_name', metavar='\b', default='ultimate_pointing_times.json', help='File name of json to be plotted. Default=ultimate_pointing_times.json')
    
    args = parser.parse_args()
    start_date  = args.start_date
    stop_date   = args.stop_date
    time_zone   = args.time_zone
    out_dir     = Path(args.out_dir)
    f_name      = Path(args.f_name)


    obs_time, obs_unix, obs_unix_end, obs_gps, obs_gps_end = time_tree(
            start_date,
            stop_date,
            time_zone)

    pointings, start_gps, stop_gps, obs_length = read_json(out_dir, f_name)

    obs_pointing(
        obs_time, 
        obs_gps, 
        obs_gps_end, 
        pointings,
        start_gps,
        stop_gps,
        start_date, 
        stop_date,
        out_dir
        )








import sys
import numpy as np
import healpy as hp
from scipy.stats import median_absolute_deviation as mad

sys.path.append('../sat_ephemeris')
from sat_ids import norad_ids

from plot_tile_maps import plot_healpix

def sat_maps(sat):
    
    f = Path(f'{map_dir}/S07XX_rf0XX_sat_maps.npz')
    f_name, _ = f.name.split('.')
    tile, ref, _, _ = f_name.split('_')
   
    pointings = ['0','2','4','41']

    # list of all possible satellites
    sat_ids = list(norad_ids.values())
    
    # load data from map .npz file
    tile_data = np.load(f, allow_pickle=True)
    tile_data = {key:tile_data[key].item() for key in tile_data}
    tile_map = tile_data['mwa_map']  
    
    for p in pointings:

        fig = plt.figure(figsize=(8,10))
        fig.suptitle(f'Satellite [{sat}]: {tile}/{ref} @ {p}', fontsize=16)
        tile_sat_med = [(np.median(i) if i != [] else np.nan ) for i in tile_data['mwa_map'][p][sat]]
        #tile_sat_scaled = np.asarray([(i - np.nanmax(tile_sat_med[:5000])) for i in tile_sat_med])
        plot_healpix(data_map=np.asarray(tile_sat_med), sub=(1,1,1), cmap=jade)
        plt.savefig(f'{out_dir}/sat_maps/{p}/{sat}_{p}_{tile}_{ref}_passes.png',bbox_inches='tight')
        plt.close()

if __name__=='__main__':
    
    import argparse
    import matplotlib
    matplotlib.use('Agg')
    from pathlib import Path
    import concurrent.futures
    import matplotlib.pyplot as plt
    
    import sys
    sys.path.append('../decode_rf_data')
    from colormap import spectral, jade, kelp
    
    # Custom spectral colormap
    jade, _ = jade()
    

    parser = argparse.ArgumentParser(description="""
        Plot healpix map of reference data
        """)
    
    parser.add_argument('--out_dir', metavar='\b', default='./../../outputs/tile_maps/',help='Output directory. Default=./../../outputs/tile_maps/')
    parser.add_argument('--map_dir', metavar='\b', default='./../../outputs/tile_maps/tile_maps_raw',help='Output directory. Default=./../../outputs/tile_maps/tile_maps_raw')
    parser.add_argument('--nside', metavar='\b', type=int,  default=32,help='Healpix Nside. Default = 32')
    
    args = parser.parse_args()
    
    out_dir = Path(args.out_dir)
    map_dir = Path(args.map_dir)
    nside   = args.nside
    
    # list of all possible satellites
    sat_ids = list(norad_ids.values())
    
    pointings = ['0','2','4', '41']
    
    map_files = [item for item in map_dir.glob('*.npz')]

    # Create output directory tree
    for p in pointings:
        Path(f'{out_dir}/sat_maps/{p}/').mkdir(parents=True, exist_ok=True)

    # plot maps for all sats, for only one tile_ref pair
    # S08XX has good time coverage
    with concurrent.futures.ProcessPoolExecutor() as executor:
        results = executor.map(sat_maps, sat_ids)
    #sat_maps(sat_ids[0])



